@@ -143,7 +143,7 @@ if ( msg0 && msg2 ) { /* L(:,k) and U(k,:) are not empty. */
 	 /* jj0 contains the look-ahead window that was updated in
 	    dlook_ahead_update.c. Now the search can continue from that point,
 	    not to start from block 0. */
-#if 0 // Sherry comment out 5/21/208
+#if 0 // Sherry comment out 5/21/2018
 	 /* Save pointers at location right after look-ahead window
 	    for later restart. */
 	 iukp0 = iukp;
@@ -404,7 +404,11 @@ if ( msg0 && msg2 ) { /* L(:,k) and U(k,:) are not empty. */
 #ifdef _OPENMP
 #pragma omp parallel default (shared) private(thread_id)
 	 {
+#ifdef _OPENMP	 
 	   thread_id = omp_get_thread_num();
+#else	   
+	   thread_id = 0;
+#endif
 
 	   /* Ideally, should organize the loop as:
 	      for (j = 0; j < nub; ++j) {
@@ -597,7 +601,11 @@ if ( msg0 && msg2 ) { /* L(:,k) and U(k,:) are not empty. */
 #ifdef _OPENMP
 #pragma omp parallel default(shared) private(thread_id)
 	{
+#ifdef _OPENMP	
 	    thread_id = omp_get_thread_num();
+#else	    
+	    thread_id = 0;
+#endif
 
 	    /* Ideally, should organize the loop as:
                for (j = 0; j < jj_cpu; ++j) {
