@@ -13,9 +13,10 @@ at the top-level directory.
 /*! @file
  * \brief Re-distribute A on the 2D process mesh.
  * <pre>
- * -- Distributed SuperLU routine (version 2.3) --
+ * -- Distributed SuperLU routine (version 7.1.1) --
  * Lawrence Berkeley National Lab, Univ. of California Berkeley.
  * October 15, 2008
+ * October 18, 2021, minor fix, v7.1.1
  * </pre>
  */
 
@@ -141,8 +142,8 @@ dReDistribute_A(SuperMatrix *A, dScalePermstruct_t *ScalePermstruct,
             ABORT("Malloc fails for ia[].");
         if ( !(aij = doubleMalloc_dist(k)) )
             ABORT("Malloc fails for aij[].");
+        ja = ia + k;
     }
-    ja = ia + k;
 
     /* Allocate temporary storage for sending/receiving the A triplets. */
     if ( procs > 1 ) {
@@ -170,9 +171,9 @@ dReDistribute_A(SuperMatrix *A, dScalePermstruct_t *ScalePermstruct,
 
       for (i = 0, j = 0, p = 0; p < procs; ++p) {
           if ( p != iam ) {
-	      ia_send[p] = &index[i];
+	      if (nnzToSend[p] > 0) ia_send[p] = &index[i];
 	      i += 2 * nnzToSend[p]; /* ia/ja indices alternate */
-	      aij_send[p] = &nzval[j];
+	      if (nnzToSend[p] > 0) aij_send[p] = &nzval[j];
 	      j += nnzToSend[p];
 	  }
       }
@@ -216,8 +217,8 @@ dReDistribute_A(SuperMatrix *A, dScalePermstruct_t *ScalePermstruct,
        NOTE: Can possibly use MPI_Alltoallv.
        ------------------------------------------------------------*/
     for (p = 0; p < procs; ++p) {
-	if ( p != iam && nnzToSend[p]>0 ) {  // cause two of the tests to hang
-	//	if ( p != iam ) {
+        if ( p != iam && nnzToSend[p] > 0 ) {
+    	//if ( p != iam ) {
 	    it = 2*nnzToSend[p];
 	    MPI_Isend( ia_send[p], it, mpi_int_t,
 		       p, iam, grid->comm, &send_req[p] );
@@ -228,8 +229,8 @@ dReDistribute_A(SuperMatrix *A, dScalePermstruct_t *ScalePermstruct,
     }
 
     for (p = 0; p < procs; ++p) {
-	if ( p != iam && nnzToRecv[p]>0 ) {
-	    //if ( p != iam ) {
+        if ( p != iam && nnzToRecv[p] > 0 ) {
+	//if ( p != iam ) {
 	    it = 2*nnzToRecv[p];
 	    MPI_Recv( itemp, it, mpi_int_t, p, p, grid->comm, &status );
 	    it = nnzToRecv[p];
@@ -248,8 +249,8 @@ dReDistribute_A(SuperMatrix *A, dScalePermstruct_t *ScalePermstruct,
     }
 
     for (p = 0; p < procs; ++p) {
-        if ( p != iam && nnzToSend[p] > 0 ) {
-	    //if ( p != iam ) {
+        if ( p != iam && nnzToSend[p] > 0 ) { // cause two of the tests to hang
+        //if ( p != iam ) {
 	    MPI_Wait( &send_req[p], &status);
 	    MPI_Wait( &send_req[procs+p], &status);
 	}
