@@ -11,7 +11,7 @@ at the top-level directory.
 
 
 /*! @file
- * \brief Performs LU factorization in parallel
+ * \brief Performs LU factorization in parallel.
  *
  * <pre>
  * -- Distributed SuperLU routine (version 6.1) --
@@ -109,7 +109,6 @@ at the top-level directory.
  */
 
 #include <math.h>
-/*#include "mkl.h"*/
 #include "superlu_ddefs.h"
 
 #ifdef GPU_ACC
@@ -129,7 +128,7 @@ at the top-level directory.
 // #define SUPERNODE_PROFILE
 
 /*
-    Name    :   BAELINE
+    Name    :   BASELINE
     Purpose : baseline to compare performance against
     Overhead : NA : this won't be used for running experiments
 */
@@ -151,18 +150,6 @@ at the top-level directory.
 #define GEMM_PADLEN 8
 
 #define PDGSTRF2 pdgstrf2_trsm
-#define PDGSTRS2 pdgstrs2_omp
-
-extern void PDGSTRF2 (superlu_dist_options_t *, int_t, int_t, double,
-                        Glu_persist_t *, gridinfo_t *, LocalLU_t *,
-                        MPI_Request *, int, SuperLUStat_t *, int *);
-#ifdef _CRAY
-extern void PDGSTRS2 (int_t, int_t, Glu_persist_t *, gridinfo_t *,
-                      LocalLU_t *, SuperLUStat_t *, _fcd, _fcd, _fcd);
-#else
-extern void PDGSTRS2 (int_t, int_t, Glu_persist_t *, gridinfo_t *,
-                      LocalLU_t *, SuperLUStat_t *);
-#endif
 
 #ifdef ISORT
 extern void isort (int_t N, int_t * ARRAY1, int_t * ARRAY2);
@@ -216,7 +203,7 @@ superlu_sort_perm (const void *arg1, const void *arg2)
  *        The norm of the original matrix A, or the scaled A if
  *        equilibration was done.
  *
- * LUstruct (input/output) LUstruct_t*
+ * LUstruct (input/output) dLUstruct_t*
  *         The data structures to store the distributed L and U factors.
  *         The following fields should be defined:
  *
@@ -227,9 +214,9 @@ superlu_sort_perm (const void *arg1, const void *arg2)
  *         xsup[s] is the leading column of the s-th supernode,
  *             supno[i] is the supernode number to which column i belongs.
  *
- *         o Llu (input/output) LocalLU_t*
+ *         o Llu (input/output) dLocalLU_t*
  *           The distributed data structures to store L and U factors.
- *           See superlu_ddefs.h for the definition of 'LocalLU_t'.
+ *           See superlu_ddefs.h for the definition of 'dLocalLU_t'.
  *
  * grid   (input) gridinfo_t*
  *        The 2D process mesh. It contains the MPI communicator, the number
@@ -254,7 +241,7 @@ superlu_sort_perm (const void *arg1, const void *arg2)
  */
 int_t
 pdgstrf(superlu_dist_options_t * options, int m, int n, double anorm,
-       LUstruct_t * LUstruct, gridinfo_t * grid, SuperLUStat_t * stat, int *info)
+       dLUstruct_t * LUstruct, gridinfo_t * grid, SuperLUStat_t * stat, int *info)
 {
 #ifdef _CRAY
     _fcd ftcs = _cptofcd ("N", strlen ("N"));
@@ -291,7 +278,7 @@ pdgstrf(superlu_dist_options_t * options, int m, int n, double anorm,
     int iinfo;
     int *ToRecv, *ToSendD, **ToSendR;
     Glu_persist_t *Glu_persist = LUstruct->Glu_persist;
-    LocalLU_t *Llu = LUstruct->Llu;
+    dLocalLU_t *Llu = LUstruct->Llu;
     superlu_scope_t *scp;
     float s_eps;
     double thresh;
@@ -571,7 +558,6 @@ pdgstrf(superlu_dist_options_t * options, int m, int n, double anorm,
 #pragma omp parallel default(shared)
     #pragma omp master
     {
-         //if (omp_get_thread_num () == 0)
         num_threads = omp_get_num_threads ();
     }
 #endif
@@ -579,7 +565,7 @@ pdgstrf(superlu_dist_options_t * options, int m, int n, double anorm,
 #if 0
     omp_loop_time = (double *) _mm_malloc (sizeof (double) * num_threads,64);
 #else
-    omp_loop_time = (double *) doubleMalloc_dist(num_threads);
+    omp_loop_time = (double *) SUPERLU_MALLOC(num_threads * sizeof(double));
 #endif
 
 #if ( PRNTlevel>=1 )
@@ -600,8 +586,8 @@ pdgstrf(superlu_dist_options_t * options, int m, int n, double anorm,
     full_u_cols = (int_t *) _mm_malloc (sizeof (int_t) * ncb,64);
     blk_ldu = (int_t *) _mm_malloc (sizeof (int_t) * ncb,64);
 #else
-    full_u_cols = SUPERLU_MALLOC(ncb * sizeof(int));
-    blk_ldu = SUPERLU_MALLOC(ncb * sizeof(int));
+    full_u_cols = SUPERLU_MALLOC((ncb+1) * sizeof(int));
+    blk_ldu = SUPERLU_MALLOC((ncb+1) * sizeof(int)); // +1 to accommodate un-even division
 #endif
 
     log_memory(2 * ncb * iword, stat);
@@ -622,7 +608,7 @@ pdgstrf(superlu_dist_options_t * options, int m, int n, double anorm,
     perm_c_supno = SUPERLU_MALLOC (2 * nsupers * sizeof (int_t));
     iperm_c_supno = perm_c_supno + nsupers;
 
-    static_schedule(options, m, n, LUstruct, grid, stat,
+    dstatic_schedule(options, m, n, LUstruct, grid, stat,
 		    perm_c_supno, iperm_c_supno, info);
 
 #if ( DEBUGlevel >= 2 )
@@ -761,7 +747,7 @@ pdgstrf(superlu_dist_options_t * options, int m, int n, double anorm,
 #pragma omp parallel for reduction(max :local_max_row_size) private(lk,lsub)
 #endif
 #endif
-    for (int i = mycol; i < nsupers; i += Pc) { /* grab my local columns */
+    for (i = mycol; i < nsupers; i += Pc) { /* grab my local columns */
         //int tpc = PCOL (i, grid);
 	lk = LBj (i, grid);
 	lsub = Lrowind_bc_ptr[lk];
@@ -780,13 +766,14 @@ pdgstrf(superlu_dist_options_t * options, int m, int n, double anorm,
          SUPERLU_MAX (max_row_size * num_threads * ldt,
                       get_max_buffer_size ());           */
 
-#ifdef GPU_ACC
-    int cublas_nb = get_cublas_nb();
-    int nstreams = get_num_cuda_streams ();
+#ifdef GPU_ACC /*-------- use GPU --------*/
+    int cublas_nb = get_cublas_nb(); // default 64
+    int nstreams = get_num_cuda_streams (); // default 8
 
-    int buffer_size  = SUPERLU_MAX(max_row_size*nstreams*cublas_nb,get_max_buffer_size());
+    int_t buffer_size  = SUPERLU_MAX(max_row_size * nstreams * cublas_nb, sp_ienv_dist(8));
+                                     //   get_max_buffer_size());
     /* array holding last column blk for each partition,
-       used in SchCompUdt--CUDA.c         */
+       used in SchCompUdt-cuda.c         */
   #if 0
     int *stream_end_col = (int_t *) _mm_malloc (sizeof (int_t) * nstreams,64);
   #else
@@ -796,8 +783,9 @@ pdgstrf(superlu_dist_options_t * options, int m, int n, double anorm,
 #else /* not to use GPU */
 
     int Threads_per_process = get_thread_per_process();
-    int buffer_size  = SUPERLU_MAX(max_row_size*Threads_per_process*ldt,get_max_buffer_size());
-#endif /* end ifdef GPU_ACC */
+    int_t buffer_size  = SUPERLU_MAX(max_row_size * Threads_per_process * ldt, sp_ienv_dist(8));
+                                     // get_max_buffer_size());
+#endif /* end ifdef GPU_ACC -----------*/
 
     int_t max_ncols = 0;
 #if 0
@@ -815,39 +803,49 @@ pdgstrf(superlu_dist_options_t * options, int m, int n, double anorm,
     int_t bigv_size = SUPERLU_MAX(max_row_size * max_ncols,
 				  (ldt*ldt + CACHELINE / dword) * num_threads);
 
-    /* bigU and bigV are either on CPU or on GPU, not both. */
-    double* bigU; /* for storing entire U(k,:) panel, prepare for GEMM.
-                      bigU has the same size either on CPU or on CPU. */
+    /* bigU and bigV are only allocated on CPU, but may be allocated as
+       page-locked memory accessible to GPU. */
+    double* bigU; /* for storing entire U(k,:) panel, prepare for GEMM. */
     double* bigV; /* for storing GEMM output matrix, i.e. update matrix.
-	              bigV is large to hold the aggregate GEMM output.*/
-    bigU = NULL;
+	              bigV is large enough to hold the aggregate GEMM output.*/
+    bigU = NULL; /* allocated only on CPU */
     bigV = NULL;
 
 #if ( PRNTlevel>=1 )
     if(!iam) {
-	printf("\t.. GEMM buffer size: max_row_size X max_ncols = %d x " IFMT "\n",
-	       max_row_size, max_ncols);
-	printf(".. BIG U size " IFMT "\t BIG V size " IFMT "\n", bigu_size, bigv_size);
-	fflush(stdout);
+        printf("\t.. MAX_BUFFER_SIZE %d set for GPU\n", sp_ienv_dist(8));
+	printf("\t.. N_GEMM: %d flops of GEMM done on CPU (1st block always on CPU)\n", sp_ienv_dist(7));
+        printf("\t.. GEMM buffer size: max_row_size X max_ncols = %d x " IFMT "\n",
+                max_row_size, max_ncols);
     }
+    printf("[%d].. BIG U size " IFMT " (on CPU)\n", iam, bigu_size);
+    fflush(stdout);
 #endif
 
-#ifdef GPU_ACC
+#ifdef GPU_ACC /*-- use GPU --*/
 
     if ( checkCuda(cudaHostAlloc((void**)&bigU,  bigu_size * sizeof(double), cudaHostAllocDefault)) )
         ABORT("Malloc fails for dgemm buffer U ");
 
+#if 0 // !!Sherry fix -- only dC on GPU uses buffer_size
     bigv_size = buffer_size;
-#if ( PRNTlevel>=1 )
-    if (!iam) printf("[%d] .. BIG V bigv_size %d, using buffer_size %d (on GPU)\n", iam, bigv_size, buffer_size);
 #endif
+
+#if ( PRNTlevel>=1 )
+    printf("[%d].. BIG V size " IFMT " (on CPU), dC buffer_size " IFMT " (on GPU)\n",
+            iam, bigv_size, buffer_size);
+    fflush(stdout);
+#endif
+
     if ( checkCuda(cudaHostAlloc((void**)&bigV, bigv_size * sizeof(double) ,cudaHostAllocDefault)) )
         ABORT("Malloc fails for dgemm buffer V");
 
-    DisplayHeader();
-
 #if ( PRNTlevel>=1 )
-    printf(" Starting with %d Cuda Streams \n",nstreams );
+    if ( iam==0 ) {
+        DisplayHeader();
+	printf(" Starting with %d Cuda Streams \n",nstreams );
+        fflush(stdout);
+    }
 #endif
 
     cublasHandle_t *handle;
@@ -876,43 +874,47 @@ pdgstrf(superlu_dist_options_t * options, int m, int n, double anorm,
         return 1;
     }
 
-    // size of B should be max_supernode_size*buffer
-
+    // size of B should be bigu_size
     cudaStat = cudaMalloc((void**)&dB, bigu_size * sizeof(double));
     if (cudaStat!= cudaSuccess) {
         fprintf(stderr, "!!!! Error in allocating B in the device %ld \n",n*k*sizeof(double));
         return 1;
     }
 
-    cudaStat = cudaMalloc((void**)&dC, buffer_size* sizeof(double) );
+    cudaStat = cudaMalloc((void**)&dC, buffer_size * sizeof(double) );
     if (cudaStat!= cudaSuccess) {
         fprintf(stderr, "!!!! Error in allocating C in the device \n" );
         return 1;
     }
 
-    stat->gpu_buffer += ( max_row_size * sp_ienv_dist(3)
-			  + bigu_size + buffer_size ) * dword;
+    stat->gpu_buffer += dword * ( max_row_size * sp_ienv_dist(3) // dA
+                                 + bigu_size                    // dB
+                                 + buffer_size );               // dC
 
-#else  /* not CUDA */
+#else  /*-------- not to use GPU --------*/
 
     // for GEMM padding 0
     j = bigu_size / ldt;
     bigu_size += (gemm_k_pad * (j + ldt + gemm_n_pad));
     bigv_size += (gemm_m_pad * (j + max_row_size + gemm_n_pad));
 
+#if ( PRNTlevel>=1 )
+    printf("[%d].. BIG V size " IFMT " (on CPU)\n", iam, bigv_size);
+    fflush(stdout);
+#endif
+
 //#ifdef __INTEL_COMPILER
 //    bigU = _mm_malloc(bigu_size * sizeof(double), 1<<12); // align at 4K page
 //    bigV = _mm_malloc(bigv_size * sizeof(double), 1<<12);
 //#else
     if ( !(bigU = doubleMalloc_dist(bigu_size)) )
         ABORT ("Malloc fails for dgemm U buffer");
-          //Maximum size of bigU= sqrt(buffsize) ?
-    // int bigv_size = 8 * ldt * ldt * num_threads;
     if ( !(bigV = doubleMalloc_dist(bigv_size)) )
         ABORT ("Malloc failed for dgemm V buffer");
 //#endif
 
-#endif /* end ifdef GPU_ACC */
+#endif 
+/*************** end ifdef GPU_ACC ****************/
 
     log_memory((bigv_size + bigu_size) * dword, stat);
 
@@ -927,11 +929,6 @@ pdgstrf(superlu_dist_options_t * options, int m, int n, double anorm,
 
 #endif
 
-#if 0 /* Sherry */
-    if (!(tempv2d = doubleCalloc_dist (2 * ((size_t) ldt) * ldt)))
-        ABORT ("Calloc fails for tempv2d[].");
-    tempU2d = tempv2d + ldt * ldt;
-#endif
     /* Sherry: (ldt + 16), avoid cache line false sharing.
        KNL cacheline size = 64 bytes = 16 int */
     iinfo = ldt + CACHELINE / sizeof(int);
@@ -978,16 +975,16 @@ pdgstrf(superlu_dist_options_t * options, int m, int n, double anorm,
 #if 0
     Remain_L_buff = (double *) _mm_malloc( sizeof(double)*(Llu->bufmax[1]),64);
     Ublock_info = (Ublock_info_t *) _mm_malloc(mcb*sizeof(Ublock_info_t),64);
-    int * Ublock_info_iukp = (int *) _mm_malloc(mcb*sizeof(int),64);
-    int * Ublock_info_rukp = (int *) _mm_malloc(mcb*sizeof(int),64);
-    int * Ublock_info_jb = (int *) _mm_malloc(mcb*sizeof(int),64);
+    /*int * Ublock_info_iukp = (int *) _mm_malloc(mcb*sizeof(int),64);
+      int * Ublock_info_rukp = (int *) _mm_malloc(mcb*sizeof(int),64);
+      int * Ublock_info_jb = (int *) _mm_malloc(mcb*sizeof(int),64); */
 #else
     j = gemm_m_pad * (ldt + max_row_size + gemm_k_pad);
     Remain_L_buff = doubleMalloc_dist(Llu->bufmax[1] + j); /* This is loose */
     Ublock_info = (Ublock_info_t *) SUPERLU_MALLOC(mcb*sizeof(Ublock_info_t));
-    int *Ublock_info_iukp = (int *) SUPERLU_MALLOC(mcb*sizeof(int));
-    int *Ublock_info_rukp = (int *) SUPERLU_MALLOC(mcb*sizeof(int));
-    int *Ublock_info_jb = (int *) SUPERLU_MALLOC(mcb*sizeof(int));
+    /*int *Ublock_info_iukp = (int *) SUPERLU_MALLOC(mcb*sizeof(int));
+      int *Ublock_info_rukp = (int *) SUPERLU_MALLOC(mcb*sizeof(int));
+      int *Ublock_info_jb = (int *) SUPERLU_MALLOC(mcb*sizeof(int)); */
 #endif
 
     long long alloc_mem = 3 * mrb * iword + mrb * sizeof(Remain_info_t)
@@ -1307,8 +1304,8 @@ pdgstrf(superlu_dist_options_t * options, int m, int n, double anorm,
 /* #pragma omp parallel */ /* Sherry -- parallel done inside pdgstrs2 */
 #endif
 			{
-                            PDGSTRS2 (kk0, kk, Glu_persist, grid, Llu,
-                                      stat);
+                            pdgstrs2_omp (kk0, kk, Glu_persist, grid, Llu,
+                                        Ublock_info, stat);
                         }
 
                         pdgstrs2_timer += SuperLU_timer_()-ttt2;
@@ -1473,7 +1470,8 @@ pdgstrf(superlu_dist_options_t * options, int m, int n, double anorm,
 /* #pragma omp parallel */ /* Sherry -- parallel done inside pdgstrs2 */
 #endif
                 {
-                    PDGSTRS2 (k0, k, Glu_persist, grid, Llu, stat);
+                    pdgstrs2_omp (k0, k, Glu_persist, grid, Llu, 
+		                    Ublock_info, stat);
                 }
                 pdgstrs2_timer += SuperLU_timer_() - ttt2;
 
@@ -1736,7 +1734,7 @@ pdgstrf(superlu_dist_options_t * options, int m, int n, double anorm,
 
 	/*******************************************************************/
 
-#ifdef GPU_ACC
+#ifdef GPU_ACC /*-- GPU --*/
 
 #include "dSchCompUdt-cuda.c"
 
@@ -1769,29 +1767,29 @@ pdgstrf(superlu_dist_options_t * options, int m, int n, double anorm,
     MPI_Reduce(&RemainGEMM_flops, &allflops, 1, MPI_DOUBLE, MPI_SUM,
 	       0, grid->comm);
     if ( iam==0 ) {
-	printf("\nInitialization time\t%8.2lf seconds\n"
+	printf("\nInitialization time\t%8.4lf seconds\n"
 	       "\t Serial: compute static schedule, allocate storage\n", InitTimer);
         printf("\n==== Time breakdown in factorization (rank 0) ====\n");
-	printf("Panel factorization \t %8.2lf seconds\n",
+	printf("Panel factorization \t %8.4lf seconds\n",
 	       pdgstrf2_timer + pdgstrs2_timer);
-	printf(".. L-panel pxgstrf2 \t %8.2lf seconds\n", pdgstrf2_timer);
-	printf(".. U-panel pxgstrs2 \t %8.2lf seconds\n", pdgstrs2_timer);
-	printf("Time in Look-ahead update \t %8.2lf seconds\n", lookaheadupdatetimer);
-        printf("Time in Schur update \t\t %8.2lf seconds\n", NetSchurUpTimer);
-        printf(".. Time to Gather L buffer\t %8.2lf  (Separate L panel by Lookahead/Remain)\n", GatherLTimer);
-        printf(".. Time to Gather U buffer\t %8.2lf \n", GatherUTimer);
+	printf(".. L-panel pxgstrf2 \t %8.4lf seconds\n", pdgstrf2_timer);
+	printf(".. U-panel pxgstrs2 \t %8.4lf seconds\n", pdgstrs2_timer);
+	printf("Time in Look-ahead update \t %8.4lf seconds\n", lookaheadupdatetimer);
+        printf("Time in Schur update \t\t %8.4lf seconds\n", NetSchurUpTimer);
+        printf(".. Time to Gather L buffer\t %8.4lf  (Separate L panel by Lookahead/Remain)\n", GatherLTimer);
+        printf(".. Time to Gather U buffer\t %8.4lf \n", GatherUTimer);
 
-        printf(".. Time in GEMM %8.2lf \n",
+        printf(".. Time in GEMM %8.4lf \n",
 	       LookAheadGEMMTimer + RemainGEMMTimer);
-        printf("\t* Look-ahead\t %8.2lf \n", LookAheadGEMMTimer);
-        printf("\t* Remain\t %8.2lf\tFlops %8.2le\tGflops %8.2lf\n",
+        printf("\t* Look-ahead\t %8.4lf \n", LookAheadGEMMTimer);
+        printf("\t* Remain\t %8.4lf\tFlops %8.4le\tGflops %8.4lf\n",
 	       RemainGEMMTimer, allflops, allflops/RemainGEMMTimer*1e-9);
-        printf(".. Time to Scatter %8.2lf \n",
+        printf(".. Time to Scatter %8.4lf \n",
 	       LookAheadScatterTimer + RemainScatterTimer);
-        printf("\t* Look-ahead\t %8.2lf \n", LookAheadScatterTimer);
-        printf("\t* Remain\t %8.2lf \n", RemainScatterTimer);
+        printf("\t* Look-ahead\t %8.4lf \n", LookAheadScatterTimer);
+        printf("\t* Remain\t %8.4lf \n", RemainScatterTimer);
 
-        printf("Total factorization time            \t: %8.2lf seconds, \n", pxgstrfTimer);
+        printf("Total factorization time            \t: %8.4lf seconds, \n", pxgstrfTimer);
         printf("--------\n");
 	printf("GEMM maximum block: %d-%d-%d\n", gemm_max_m, gemm_max_k, gemm_max_n);
     }
@@ -1920,9 +1918,9 @@ pdgstrf(superlu_dist_options_t * options, int m, int n, double anorm,
 		  Llu->bufmax[1] * dword), stat );
 
     SUPERLU_FREE(Ublock_info);
-    SUPERLU_FREE(Ublock_info_iukp);
-    SUPERLU_FREE(Ublock_info_rukp);
-    SUPERLU_FREE(Ublock_info_jb);
+    /*SUPERLU_FREE(Ublock_info_iukp);
+      SUPERLU_FREE(Ublock_info_rukp);
+      SUPERLU_FREE(Ublock_info_jb);  */
 
 
 #if ( PROFlevel>=1 )
