@@ -107,7 +107,7 @@ _fcd ftcs3;
  *
  * Note
  * ====
- *   This routine can only be called after the routine pxgstrs_init(),
+ *   This routine can only be called after the routine pdgstrs_init(),
  *   in which the structures of the send and receive buffers are set up.
  *
  * Arguments
@@ -135,14 +135,14 @@ _fcd ftcs3;
  * x      (output) double*
  *        The solution vector. It is valid only on the diagonal processes.
  *
- * ScalePermstruct (input) ScalePermstruct_t*
+ * ScalePermstruct (input) dScalePermstruct_t*
  *        The data structure to store the scaling and permutation vectors
  *        describing the transformations performed to the original matrix A.
  *
  * grid   (input) gridinfo_t*
  *        The 2D process mesh.
  *
- * SOLVEstruct (input) SOLVEstruct_t*
+ * SOLVEstruct (input) dSOLVEstruct_t*
  *        Contains the information for the communication during the
  *        solution phase.
  *
@@ -154,9 +154,9 @@ _fcd ftcs3;
 int_t
 pdReDistribute_B_to_X(double *B, int_t m_loc, int nrhs, int_t ldb,
                       int_t fst_row, int_t *ilsum, double *x,
-		      ScalePermstruct_t *ScalePermstruct,
+		      dScalePermstruct_t *ScalePermstruct,
 		      Glu_persist_t *Glu_persist,
-		      gridinfo_t *grid, SOLVEstruct_t *SOLVEstruct)
+		      gridinfo_t *grid, dSOLVEstruct_t *SOLVEstruct)
 {
     int  *SendCnt, *SendCnt_nrhs, *RecvCnt, *RecvCnt_nrhs;
     int  *sdispls, *sdispls_nrhs, *rdispls, *rdispls_nrhs;
@@ -198,7 +198,6 @@ pdReDistribute_B_to_X(double *B, int_t m_loc, int nrhs, int_t ldb,
     /* ------------------------------------------------------------
        NOW COMMUNICATE THE ACTUAL DATA.
        ------------------------------------------------------------*/
-
 	if(procs==1){ // faster memory copy when procs=1
 
 #ifdef _OPENMP
@@ -402,7 +401,7 @@ pdReDistribute_B_to_X(double *B, int_t m_loc, int nrhs, int_t ldb,
  *
  * Note
  * ====
- *   This routine can only be called after the routine pxgstrs_init(),
+ *   This routine can only be called after the routine pdgstrs_init(),
  *   in which the structures of the send and receive buffers are set up.
  * </pre>
  */
@@ -410,9 +409,9 @@ pdReDistribute_B_to_X(double *B, int_t m_loc, int nrhs, int_t ldb,
 int_t
 pdReDistribute_X_to_B(int_t n, double *B, int_t m_loc, int_t ldb, int_t fst_row,
 		      int_t nrhs, double *x, int_t *ilsum,
-		      ScalePermstruct_t *ScalePermstruct,
+		      dScalePermstruct_t *ScalePermstruct,
 		      Glu_persist_t *Glu_persist, gridinfo_t *grid,
-		      SOLVEstruct_t *SOLVEstruct)
+		      dSOLVEstruct_t *SOLVEstruct)
 {
     int_t  i, ii, irow, j, jj, k, knsupc, nsupers, l, lk;
     int_t  *xsup, *supno;
@@ -455,7 +454,6 @@ pdReDistribute_X_to_B(int_t n, double *B, int_t m_loc, int_t ldb, int_t fst_row,
 
 
 	if(procs==1){ //faster memory copy when procs=1
-
 #ifdef _OPENMP
 #pragma omp parallel default (shared)
 #endif
@@ -646,12 +644,12 @@ pdReDistribute_X_to_B(int_t n, double *B, int_t m_loc, int_t ldb, int_t fst_row,
  * </pre>
  */
 void
-pdCompute_Diag_Inv(int_t n, LUstruct_t *LUstruct,gridinfo_t *grid,
+pdCompute_Diag_Inv(int_t n, dLUstruct_t *LUstruct,gridinfo_t *grid,
                    SuperLUStat_t *stat, int *info)
 {
 #ifdef SLU_HAVE_LAPACK
     Glu_persist_t *Glu_persist = LUstruct->Glu_persist;
-    LocalLU_t *Llu = LUstruct->Llu;
+    dLocalLU_t *Llu = LUstruct->Llu;
 
     double *lusup;
     double *recvbuf, *tempv;
@@ -787,11 +785,11 @@ pdCompute_Diag_Inv(int_t n, LUstruct_t *LUstruct,gridinfo_t *grid,
  * n      (input) int (global)
  *        The order of the system of linear equations.
  *
- * LUstruct (input) LUstruct_t*
+ * LUstruct (input) dLUstruct_t*
  *        The distributed data structures storing L and U factors.
  *        The L and U factors are obtained from PDGSTRF for
  *        the possibly scaled and permuted matrix A.
- *        See superlu_ddefs.h for the definition of 'LUstruct_t'.
+ *        See superlu_ddefs.h for the definition of 'dLUstruct_t'.
  *        A may be scaled and permuted into A1, so that
  *        A1 = Pc*Pr*diag(R)*A*diag(C)*Pc^T = L*U
  *
@@ -822,7 +820,7 @@ pdCompute_Diag_Inv(int_t n, LUstruct_t *LUstruct,gridinfo_t *grid,
  * nrhs   (input) int (global)
  *        Number of right-hand sides.
  *
- * SOLVEstruct (input) SOLVEstruct_t* (global)
+ * SOLVEstruct (input) dSOLVEstruct_t* (global)
  *        Contains the information for the communication during the
  *        solution phase.
  *
@@ -837,15 +835,15 @@ pdCompute_Diag_Inv(int_t n, LUstruct_t *LUstruct,gridinfo_t *grid,
  */
 
 void
-pdgstrs(int_t n, LUstruct_t *LUstruct,
-	ScalePermstruct_t *ScalePermstruct,
+pdgstrs(int_t n, dLUstruct_t *LUstruct,
+	dScalePermstruct_t *ScalePermstruct,
 	gridinfo_t *grid, double *B,
 	int_t m_loc, int_t fst_row, int_t ldb, int nrhs,
-	SOLVEstruct_t *SOLVEstruct,
+	dSOLVEstruct_t *SOLVEstruct,
 	SuperLUStat_t *stat, int *info)
 {
     Glu_persist_t *Glu_persist = LUstruct->Glu_persist;
-    LocalLU_t *Llu = LUstruct->Llu;
+    dLocalLU_t *Llu = LUstruct->Llu;
     double alpha = 1.0;
 	double beta = 0.0;
     double zero = 0.0;
@@ -954,8 +952,8 @@ pdgstrs(int_t n, LUstruct_t *LUstruct,
     	int nbrow;
     int_t  ik, rel, idx_r, jb, nrbl, irow, pc,iknsupc;
     int_t  lptr1_tmp, idx_i, idx_v,m;
-    	int_t ready;
-    	static int thread_id;
+    int_t ready;
+    int thread_id = 0;
     yes_no_t empty;
     int_t sizelsum,sizertemp,aln_d,aln_i;
     aln_d = ceil(CACHELINE/(double)dword);
@@ -964,9 +962,6 @@ pdgstrs(int_t n, LUstruct_t *LUstruct,
 
 	maxsuper = sp_ienv_dist(3);
 
-#ifdef _OPENMP
-	#pragma omp threadprivate(thread_id)
-#endif
 
 #ifdef _OPENMP
 #pragma omp parallel default(shared)
@@ -974,8 +969,9 @@ pdgstrs(int_t n, LUstruct_t *LUstruct,
     	if (omp_get_thread_num () == 0) {
     		num_thread = omp_get_num_threads ();
     	}
-		thread_id = omp_get_thread_num ();
     }
+#else
+	num_thread=1;
 #endif
 
 #if ( PRNTlevel>=1 )
@@ -1063,10 +1059,11 @@ pdgstrs(int_t n, LUstruct_t *LUstruct,
 #ifdef _OPENMP
     if ( !(lsum = (double*)SUPERLU_MALLOC(sizelsum*num_thread * sizeof(double))))
 	ABORT("Malloc fails for lsum[].");
-#pragma omp parallel default(shared) private(ii)
+#pragma omp parallel default(shared) private(ii) 
     {
+	int thread_id = omp_get_thread_num(); //mjc
 	for (ii=0; ii<sizelsum; ii++)
-    	lsum[thread_id*sizelsum+ii]=zero;
+    	    lsum[thread_id*sizelsum+ii]=zero;
     }
 #else
     if ( !(lsum = (double*)SUPERLU_MALLOC(sizelsum*num_thread * sizeof(double))))
@@ -1084,6 +1081,7 @@ pdgstrs(int_t n, LUstruct_t *LUstruct,
 #ifdef _OPENMP
 #pragma omp parallel default(shared) private(ii)
     {
+	int thread_id=omp_get_thread_num();
 	for ( ii=0; ii<sizertemp; ii++ )
 		rtemp[thread_id*sizertemp+ii]=zero;
     }
@@ -1120,9 +1118,6 @@ pdgstrs(int_t n, LUstruct_t *LUstruct,
 #endif
 
     /* Set up the headers in lsum[]. */
-#ifdef _OPENMP
-	#pragma omp simd lastprivate(krow,lk,il)
-#endif
     for (k = 0; k < nsupers; ++k) {
 	krow = PROW( k, grid );
 	if ( myrow == krow ) {
@@ -1192,9 +1187,6 @@ if(procs==1){
 }
 
 
-#ifdef _OPENMP
-#pragma omp simd
-#endif
 	for (i = 0; i < nlb; ++i) fmod[i*aln_i] += frecv[i];
 
 	if ( !(recvbuf_BC_fwd = (double*)SUPERLU_MALLOC(maxrecvsz*(nfrecvx+1) * sizeof(double))) )  // this needs to be optimized for 1D row mapping
@@ -1240,8 +1232,12 @@ if(procs==1){
 
 #ifdef _OPENMP
 #pragma omp parallel default (shared)
+{
+int thread_id = omp_get_thread_num();
+#else
+{
+thread_id=0;
 #endif
-	{
 		{
 
             if (Llu->inv == 1) { /* Diagonal is inverted. */
@@ -1249,13 +1245,13 @@ if(procs==1){
 #ifdef _OPENMP
 #pragma	omp	for firstprivate(nrhs,beta,alpha,x,rtemp,ldalsum) private (ii,k,knsupc,lk,luptr,lsub,nsupr,lusup,t1,t2,Linv,i,lib,rtemp_loc,nleaf_send_tmp) nowait
 #endif
-			for (jj=0;jj<nleaf;jj++){
-				k=leafsups[jj];
+		for (jj=0;jj<nleaf;jj++){
+		    k=leafsups[jj];
 
-				// #ifdef _OPENMP
-				// #pragma	omp	task firstprivate (k,nrhs,beta,alpha,x,rtemp,ldalsum) private (ii,knsupc,lk,luptr,lsub,nsupr,lusup,thread_id,t1,t2,Linv,i,lib,rtemp_loc)
-				// #endif
-				{
+// #ifdef _OPENMP
+// #pragma omp task firstprivate (k,nrhs,beta,alpha,x,rtemp,ldalsum) private (ii,knsupc,lk,luptr,lsub,nsupr,lusup,thread_id,t1,t2,Linv,i,lib,rtemp_loc)
+// #endif
+   		    {
 
 #if ( PROFlevel>=1 )
 					TIC(t1);
@@ -1288,9 +1284,6 @@ if(procs==1){
 							&knsupc, &beta, rtemp_loc, &knsupc );
 #endif
 
-				#ifdef _OPENMP
-					#pragma omp simd
-				#endif
 					for (i=0 ; i<knsupc*nrhs ; i++){
 						x[ii+i] = rtemp_loc[i];
 					}
@@ -1366,12 +1359,6 @@ if(procs==1){
 					lusup, &nsupr, &x[ii], &knsupc);
 #endif
 
-		// for (i=0 ; i<knsupc*nrhs ; i++){
-		// printf("x_l: %f\n",x[ii+i]);
-		// fflush(stdout);
-		// }
-
-
 #if ( PROFlevel>=1 )
 		    TOC(t2, t1);
 		    stat_loc[thread_id]->utime[SOL_TRSM] += t2;
@@ -1409,39 +1396,44 @@ if(procs==1){
 
 #ifdef _OPENMP
 #pragma omp parallel default (shared)
+	{
+#else
+	{
 #endif
-		{
-
 
 #ifdef _OPENMP
 #pragma omp master
 #endif
-				{
+		    {
 
 #ifdef _OPENMP
-#pragma	omp	taskloop private (k,ii,lk) num_tasks(num_thread*8) nogroup
+#pragma	omp taskloop private (k,ii,lk,thread_id) num_tasks(num_thread*8) nogroup
 #endif
 
-					for (jj=0;jj<nleaf;jj++){
-						k=leafsups[jj];
+			for (jj=0;jj<nleaf;jj++){
+			    k=leafsups[jj];
 
-						{
-							/* Diagonal process */
-							lk = LBi( k, grid );
-							ii = X_BLK( lk );
-							/*
-							 * Perform local block modifications: lsum[i] -= L_i,k * X[k]
-							 */
-							dlsum_fmod_inv(lsum, x, &x[ii], rtemp, nrhs, k,
-									fmod, xsup, grid, Llu,
-									stat_loc, leaf_send, &nleaf_send,sizelsum,sizertemp,0,maxsuper,thread_id,num_thread);
-						}
+			    {
 
-						// } /* if diagonal process ... */
-					} /* for k ... */
-				}
+#ifdef _OPENMP
+				thread_id=omp_get_thread_num();
+#else
+				thread_id=0;
+#endif
 
-			}
+				/* Diagonal process */
+				lk = LBi( k, grid );
+				ii = X_BLK( lk );
+				/*
+				 * Perform local block modifications: lsum[i] -= L_i,k * X[k]
+				 */
+				dlsum_fmod_inv(lsum, x, &x[ii], rtemp, nrhs, k, fmod, xsup, grid, Llu, stat_loc, leaf_send, &nleaf_send,sizelsum,sizertemp,0,maxsuper,thread_id,num_thread);
+			    }
+
+			} /* for jj ... */
+		    }
+
+		}
 
 			for (i=0;i<nleaf_send;i++){
 				lk = leaf_send[i*aln_i];
@@ -1470,8 +1462,13 @@ if(procs==1){
 
 #ifdef _OPENMP
 #pragma omp parallel default (shared)
-#endif
 			{
+	int thread_id = omp_get_thread_num();
+#else
+	{
+	thread_id=0;
+#endif
+
 #ifdef _OPENMP
 #pragma omp master
 #endif
@@ -1575,17 +1572,11 @@ if(procs==1){
 											// ii = X_BLK( lk );
 											knsupc = SuperSize( k );
 											for (ii=1;ii<num_thread;ii++)
-											#ifdef _OPENMP
-												#pragma omp simd
-											#endif
 												for (jj=0;jj<knsupc*nrhs;jj++)
 													lsum[il + jj ] += lsum[il + jj + ii*sizelsum];
 
 											ii = X_BLK( lk );
 											RHS_ITERATE(j)
-												#ifdef _OPENMP
-													#pragma omp simd
-												#endif
 												for (i = 0; i < knsupc; ++i)
 													x[i + ii + j*knsupc] += lsum[i + il + j*knsupc ];
 
@@ -1614,9 +1605,6 @@ if(procs==1){
 														&alpha, Linv, &knsupc, &x[ii],
 														&knsupc, &beta, rtemp_loc, &knsupc );
 #endif
-												#ifdef _OPENMP
-													#pragma omp simd
-												#endif
 												for (i=0 ; i<knsupc*nrhs ; i++){
 													x[ii+i] = rtemp_loc[i];
 												}
@@ -1640,6 +1628,7 @@ if(procs==1){
 #endif
 
 											stat_loc[thread_id]->ops[SOLVE] += knsupc * (knsupc - 1) * nrhs;
+
 #if ( DEBUGlevel>=2 )
 											printf("(%2d) Solve X[%2d]\n", iam, k);
 #endif
@@ -1676,9 +1665,6 @@ if(procs==1){
 										knsupc = SuperSize( k );
 
 										for (ii=1;ii<num_thread;ii++)
-											#ifdef _OPENMP
-												#pragma omp simd
-											#endif
 											for (jj=0;jj<knsupc*nrhs;jj++)
 												lsum[il + jj] += lsum[il + jj + ii*sizelsum];
 										RdTree_forwardMessageSimple(LRtree_ptr[lk],&lsum[il-LSUM_H],RdTree_GetMsgSize(LRtree_ptr[lk],'d')*nrhs+LSUM_H,'d');
@@ -1693,7 +1679,7 @@ if(procs==1){
 				} /* while not finished ... */
 
 			}
-		}
+		} // end of parallel 
 
 #if ( PRNTlevel>=2 )
 		t = SuperLU_timer_() - t;
@@ -1790,13 +1776,11 @@ if(procs==1){
 
 #pragma omp parallel default(shared) private(ii)
 	{
+		int thread_id = omp_get_thread_num();
 		for(ii=0;ii<sizelsum;ii++)
 			lsum[thread_id*sizelsum+ii]=zero;
 	}
     /* Set up the headers in lsum[]. */
-#ifdef _OPENMP
-	#pragma omp simd lastprivate(krow,lk,il)
-#endif
     for (k = 0; k < nsupers; ++k) {
 	krow = PROW( k, grid );
 	if ( myrow == krow ) {
@@ -1906,9 +1890,6 @@ if(procs==1){
 		}
 	}
 
-	#ifdef _OPENMP
-	#pragma omp simd
-	#endif
 	for (i = 0; i < nlb; ++i) bmod[i*aln_i] += brecv[i];
 	// for (i = 0; i < nlb; ++i)printf("bmod[i]: %5d\n",bmod[i]);
 
@@ -1946,14 +1927,16 @@ if(procs==1){
 
 #ifdef _OPENMP
 #pragma omp parallel default (shared)
-#endif
 	{
+#else
+	{
+#endif
 #ifdef _OPENMP
 #pragma omp master
 #endif
 		{
 #ifdef _OPENMP
-#pragma	omp	taskloop firstprivate (nrhs,beta,alpha,x,rtemp,ldalsum) private (ii,jj,k,knsupc,lk,luptr,lsub,nsupr,lusup,t1,t2,Uinv,i,lib,rtemp_loc,nroot_send_tmp) nogroup
+#pragma	omp	taskloop firstprivate (nrhs,beta,alpha,x,rtemp,ldalsum) private (ii,jj,k,knsupc,lk,luptr,lsub,nsupr,lusup,t1,t2,Uinv,i,lib,rtemp_loc,nroot_send_tmp,thread_id) nogroup
 #endif
 		for (jj=0;jj<nroot;jj++){
 			k=rootsups[jj];
@@ -1961,6 +1944,11 @@ if(procs==1){
 #if ( PROFlevel>=1 )
 			TIC(t1);
 #endif
+#ifdef _OPENMP
+			thread_id=omp_get_thread_num();
+#else
+			thread_id=0;
+#endif
 
 			rtemp_loc = &rtemp[sizertemp* thread_id];
 
@@ -1993,9 +1981,6 @@ if(procs==1){
 						&alpha, Uinv, &knsupc, &x[ii],
 						&knsupc, &beta, rtemp_loc, &knsupc );
 #endif
-				#ifdef _OPENMP
-					#pragma omp simd
-				#endif
 				for (i=0 ; i<knsupc*nrhs ; i++){
 					x[ii+i] = rtemp_loc[i];
 				}
@@ -2011,15 +1996,6 @@ if(procs==1){
 						lusup, &nsupr, &x[ii], &knsupc);
 #endif
 			}
-			// for (i=0 ; i<knsupc*nrhs ; i++){
-			// printf("x_u: %f\n",x[ii+i]);
-			// fflush(stdout);
-			// }
-
-			// for (i=0 ; i<knsupc*nrhs ; i++){
-				// printf("x: %f\n",x[ii+i]);
-				// fflush(stdout);
-			// }
 
 #if ( PROFlevel>=1 )
 			TOC(t2, t1);
@@ -2050,21 +2026,27 @@ if(procs==1){
 
 #ifdef _OPENMP
 #pragma omp parallel default (shared)
-#endif
 	{
+#else
+	{
+#endif
 #ifdef _OPENMP
 #pragma omp master
 #endif
 		{
 #ifdef _OPENMP
-#pragma	omp	taskloop private (ii,jj,k,lk) nogroup
+#pragma	omp	taskloop private (ii,jj,k,lk,thread_id) nogroup
 #endif
 		for (jj=0;jj<nroot;jj++){
 			k=rootsups[jj];
 			lk = LBi( k, grid ); /* Local block number, row-wise. */
 			ii = X_BLK( lk );
 			lk = LBj( k, grid ); /* Local block number, column-wise */
-
+#ifdef _OPENMP
+			thread_id=omp_get_thread_num();
+#else
+			thread_id=0;
+#endif
 			/*
 			 * Perform local block modifications: lsum[i] -= U_i,k * X[k]
 			 */
@@ -2099,8 +2081,12 @@ for (i=0;i<nroot_send;i++){
 
 #ifdef _OPENMP
 #pragma omp parallel default (shared)
-#endif
 	{
+	int thread_id=omp_get_thread_num();
+#else
+	{
+	thread_id=0;
+#endif
 #ifdef _OPENMP
 #pragma omp master
 #endif
@@ -2164,9 +2150,6 @@ for (i=0;i<nroot_send;i++){
 				tempv = &recvbuf0[LSUM_H];
 				il = LSUM_BLK( lk );
 				RHS_ITERATE(j) {
-					#ifdef _OPENMP
-						#pragma omp simd
-					#endif
 					for (i = 0; i < knsupc; ++i)
 						lsum[i + il + j*knsupc + thread_id*sizelsum] += tempv[i + j*knsupc];
 
@@ -2182,19 +2165,13 @@ for (i=0;i<nroot_send;i++){
 
 						knsupc = SuperSize( k );
 						for (ii=1;ii<num_thread;ii++)
-							#ifdef _OPENMP
-								#pragma omp simd
-							#endif
 							for (jj=0;jj<knsupc*nrhs;jj++)
 								lsum[il+ jj ] += lsum[il + jj + ii*sizelsum];
 
 						ii = X_BLK( lk );
 						RHS_ITERATE(j)
-							#ifdef _OPENMP
-								#pragma omp simd
-							#endif
 							for (i = 0; i < knsupc; ++i)
-								x[i + ii + j*knsupc] += lsum[i + il + j*knsupc ];
+							    x[i + ii + j*knsupc] += lsum[i + il + j*knsupc ];
 
 						lk = LBj( k, grid ); /* Local block number, column-wise. */
 						lsub = Lrowind_bc_ptr[lk];
@@ -2219,9 +2196,6 @@ for (i=0;i<nroot_send;i++){
 									&knsupc, &beta, rtemp_loc, &knsupc );
 #endif
 
-							#ifdef _OPENMP
-								#pragma omp simd
-							#endif
 							for (i=0 ; i<knsupc*nrhs ; i++){
 								x[ii+i] = rtemp_loc[i];
 							}
@@ -2270,9 +2244,6 @@ for (i=0;i<nroot_send;i++){
 						knsupc = SuperSize( k );
 
 						for (ii=1;ii<num_thread;ii++)
-							#ifdef _OPENMP
-								#pragma omp simd
-							#endif
 							for (jj=0;jj<knsupc*nrhs;jj++)
 								lsum[il+ jj ] += lsum[il + jj + ii*sizelsum];
 
