@@ -323,7 +323,7 @@ at the top-level directory.
  *           = LargeDiag_MC64: use the Duff/Koster algorithm to permute rows
  *                        of the original matrix to make the diagonal large
  *                        relative to the off-diagonal.
- *           = LargeDiag_APWM: use the parallel approximate-weight perfect
+ *           = LargeDiag_HPWM: use the parallel approximate-weight perfect
  *                        matching to permute rows of the original matrix
  *                        to make the diagonal large relative to the
  *                        off-diagonal.
@@ -368,7 +368,7 @@ at the top-level directory.
  *           If all the above condition are true, the LU decomposition is
  *           performed on the matrix Pc*Pr*diag(R)*A*diag(C)*Pc^T.
  *
- * ScalePermstruct (input/output) ScalePermstruct_t* (global)
+ * ScalePermstruct (input/output) zScalePermstruct_t* (global)
  *         The data structure to store the scaling and permutation vectors
  *         describing the transformations performed to the matrix A.
  *         It contains the following fields:
@@ -405,7 +405,7 @@ at the top-level directory.
  *           of Pc*A'*A*Pc'; perm_c is not changed if the elimination tree
  *           is already in postorder.
  *
- *         o R (double*) dimension (A->nrow)
+ *         o R (double *) dimension (A->nrow)
  *           The row scale factors for A.
  *           If DiagScale = ROW or BOTH, A is multiplied on the left by
  *                          diag(R).
@@ -413,7 +413,7 @@ at the top-level directory.
  *           If options->Fact = FACTORED or SamePattern_SameRowPerm, R is
  *           an input argument; otherwise, R is an output argument.
  *
- *         o C (double*) dimension (A->ncol)
+ *         o C (double *) dimension (A->ncol)
  *           The column scale factors for A.
  *           If DiagScale = COL or BOTH, A is multiplied on the right by
  *                          diag(C).
@@ -443,7 +443,7 @@ at the top-level directory.
  *         Grid can be initialized by subroutine SUPERLU_GRIDINIT.
  *         See superlu_zdefs.h for the definition of 'gridinfo_t'.
  *
- * LUstruct (input/output) LUstruct_t*
+ * LUstruct (input/output) zLUstruct_t*
  *         The data structures to store the distributed L and U factors.
  *         It contains the following fields:
  *
@@ -466,17 +466,17 @@ at the top-level directory.
  *	       xsup[s] is the leading column of the s-th supernode,
  *             supno[i] is the supernode number to which column i belongs.
  *
- *         o Llu (LocalLU_t*) (local)
+ *         o Llu (zLocalLU_t*) (local)
  *           The distributed data structures to store L and U factors.
- *           See superlu_zdefs.h for the definition of 'LocalLU_t'.
+ *           See superlu_zdefs.h for the definition of 'zLocalLU_t'.
  *
- * SOLVEstruct (input/output) SOLVEstruct_t*
+ * SOLVEstruct (input/output) zSOLVEstruct_t*
  *         The data structure to hold the communication pattern used
  *         in the phases of triangular solution and iterative refinement.
  *         This pattern should be initialized only once for repeated solutions.
  *         If options->SolveInitialized = YES, it is an input argument.
  *         If options->SolveInitialized = NO and nrhs != 0, it is an output
- *         argument. See superlu_zdefs.h for the definition of 'SOLVEstruct_t'.
+ *         argument. See superlu_zdefs.h for the definition of 'zSOLVEstruct_t'.
  *
  * berr    (output) double*, dimension (nrhs) (global)
  *         The componentwise relative backward error of each solution
@@ -489,6 +489,7 @@ at the top-level directory.
  *
  * info    (output) int*
  *         = 0: successful exit
+ *         < 0: if info = -i, the i-th argument had an illegal value  
  *         > 0: if info = i, and i is
  *             <= A->ncol: U(i,i) is exactly zero. The factorization has
  *                been completed, but the factor U is exactly singular,
@@ -502,9 +503,9 @@ at the top-level directory.
 
 void
 pzgssvx(superlu_dist_options_t *options, SuperMatrix *A,
-	ScalePermstruct_t *ScalePermstruct,
+	zScalePermstruct_t *ScalePermstruct,
 	doublecomplex B[], int ldb, int nrhs, gridinfo_t *grid,
-	LUstruct_t *LUstruct, SOLVEstruct_t *SOLVEstruct, double *berr,
+	zLUstruct_t *LUstruct, zSOLVEstruct_t *SOLVEstruct, double *berr,
 	SuperLUStat_t *stat, int *info)
 {
     NRformat_loc *Astore;
@@ -586,13 +587,13 @@ pzgssvx(superlu_dist_options_t *options, SuperMatrix *A,
     /* Test the input parameters. */
     *info = 0;
     Fact = options->Fact;
-    if ( Fact < 0 || Fact > FACTORED )
+    if ( Fact < DOFACT || Fact > FACTORED )
 	*info = -1;
-    else if ( options->RowPerm < 0 || options->RowPerm > MY_PERMR )
+    else if ( options->RowPerm < NOROWPERM || options->RowPerm > MY_PERMR )
 	*info = -1;
-    else if ( options->ColPerm < 0 || options->ColPerm > MY_PERMC )
+    else if ( options->ColPerm < NATURAL || options->ColPerm > MY_PERMC )
 	*info = -1;
-    else if ( options->IterRefine < 0 || options->IterRefine > SLU_EXTRA )
+    else if ( options->IterRefine < NOREFINE || options->IterRefine > SLU_EXTRA )
 	*info = -1;
     else if ( options->IterRefine == SLU_EXTRA ) {
 	*info = -1;
@@ -663,6 +664,7 @@ pzgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 		    ABORT("Malloc fails for R[].");
 		ScalePermstruct->R = R;
 		break;
+	    default: break;
 	}
     }
 
@@ -912,9 +914,9 @@ pzgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 		        if ( !iam ) printf("\t product of diagonal %e\n", dprod);
 	            }
 #endif
-                } else { /* use largeDiag_AWPM */
+                } else { /* use LargeDiag_HWPM */
 #ifdef HAVE_COMBBLAS
-		    c2cpp_GetAWPM(A, grid, ScalePermstruct);
+		    z_c2cpp_GetHWPM(A, grid, ScalePermstruct);
 #else
 		    if ( iam == 0 ) {
 		        printf("CombBLAS is not available\n"); fflush(stdout);
@@ -1059,7 +1061,7 @@ pzgssvx(superlu_dist_options_t *options, SuperMatrix *A,
                    the nonzero data structures for L & U. */
 #if ( PRNTlevel>=1 )
                 if ( !iam ) {
-		    printf(".. symbfact(): relax " IFMT ", maxsuper " IFMT ", fill " IFMT "\n",
+		    printf(".. symbfact(): relax %d, maxsuper %d, fill %d\n",
 		          sp_ienv_dist(2), sp_ienv_dist(3), sp_ienv_dist(6));
 		    fflush(stdout);
 	        }
@@ -1081,10 +1083,10 @@ pzgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 		    	printf("\tNo of supers " IFMT "\n", Glu_persist->supno[n-1]+1);
 		    	printf("\tSize of G(L) " IFMT "\n", Glu_freeable->xlsub[n]);
 		    	printf("\tSize of G(U) " IFMT "\n", Glu_freeable->xusub[n]);
-		    	printf("\tint %d, short %d, float %d, double %d\n",
-			       (int) sizeof(int_t), (int) sizeof(short),
-        		       (int) sizeof(float), (int) sizeof(double));
-		    	printf("\tSYMBfact (MB):\tL\\U %.2f\ttotal %.2f\texpansions " IFMT "\n",
+		    	printf("\tint %lu, short %lu, float %lu, double %lu\n",
+			        sizeof(int_t), sizeof(short),
+        		        sizeof(float), sizeof(double));
+		    	printf("\tSYMBfact (MB):\tL\\U %.2f\ttotal %.2f\texpansions %d\n",
 			   	symb_mem_usage.for_lu*1e-6,
 			   	symb_mem_usage.total*1e-6,
 			   	symb_mem_usage.expansions);
@@ -1228,11 +1230,6 @@ pzgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 
 	MPI_Comm_rank( MPI_COMM_WORLD, &iam_g );
 
-    if (!iam_g) {
-	print_options_dist(options);
-	fflush(stdout);
-    }
-
     printf(".. Ainfo mygid %5d   mysid %5d   nnz_loc " IFMT "  sum_loc  %e lsum_loc   %e nnz "IFMT " nnzLU %ld sum %e  lsum %e  N "IFMT "\n", iam_g,iam,Astore->rowptr[Astore->m_loc],asum.r+asum.i, lsum.r+lsum.i, nnz_tot,nnzLU,asum_tot.r+asum_tot.i,lsum_tot.r+lsum_tot.i,A->ncol);
 	fflush(stdout);
 #endif
@@ -1323,7 +1320,8 @@ pzgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 		       avg * 1e-6,
 		       avg / grid->nprow / grid->npcol * 1e-6,
 		       max * 1e-6);
-		printf("**************************************************\n");
+		printf("**************************************************\n\n");
+		printf("** number of Tiny Pivots: %8d\n\n", stat->TinyPivots);
 		fflush(stdout);
             }
 	} /* end printing stats */
@@ -1343,7 +1341,7 @@ pzgssvx(superlu_dist_options_t *options, SuperMatrix *A,
     /* Need to revisit: Why the following is not good enough for X-to-B
        distribution -- inv_perm_c changed */
 	pxgstrs_finalize(SOLVEstruct->gstrs_comm);
-	pxgstrs_init(A->ncol, m_loc, nrhs, fst_row, perm_r, perm_c, grid,
+	pzgstrs_init(A->ncol, m_loc, nrhs, fst_row, perm_r, perm_c, grid,
 	             LUstruct->Glu_persist, SOLVEstruct);
 #endif
 
@@ -1433,7 +1431,7 @@ pzgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 	    int_t *it;
             int_t *colind_gsmv = SOLVEstruct->A_colind_gsmv;
 	          /* This was allocated and set to NULL in zSolveInit() */
-	    SOLVEstruct_t *SOLVEstruct1;  /* Used by refinement. */
+	    zSOLVEstruct_t *SOLVEstruct1;  /* Used by refinement. */
 
 	    t = SuperLU_timer_();
 	    if ( options->RefineInitialized == NO || Fact == DOFACT ) {
@@ -1481,8 +1479,8 @@ pzgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 			is different than the solve with nrhs RHS.
 			So we use SOLVEstruct1 for the refinement step.
 		     */
-	        if ( !(SOLVEstruct1 = (SOLVEstruct_t *)
-		                       SUPERLU_MALLOC(sizeof(SOLVEstruct_t))) )
+	        if ( !(SOLVEstruct1 = (zSOLVEstruct_t *)
+		                       SUPERLU_MALLOC(sizeof(zSOLVEstruct_t))) )
 		    ABORT("Malloc fails for SOLVEstruct1");
 	        /* Copy the same stuff */
 	        SOLVEstruct1->row_to_proc = SOLVEstruct->row_to_proc;
@@ -1497,7 +1495,7 @@ pzgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 		if ( !(SOLVEstruct1->gstrs_comm = (pxgstrs_comm_t *)
 		       SUPERLU_MALLOC(sizeof(pxgstrs_comm_t))) )
 		    ABORT("Malloc fails for gstrs_comm[]");
-		pxgstrs_init(n, m_loc, 1, fst_row, perm_r, perm_c, grid,
+		pzgstrs_init(n, m_loc, 1, fst_row, perm_r, perm_c, grid,
 			     Glu_persist, SOLVEstruct1);
 	    }
 
@@ -1571,6 +1569,7 @@ pzgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 	    case COL:
 		SUPERLU_FREE(R);
 		break;
+	    default: break;
 	}
     }
 
