@@ -13,14 +13,15 @@ at the top-level directory.
  * \brief Perform local block modifications: lsum[i] -= L_i,k * X[k]
  *
  * <pre>
- * -- Distributed SuperLU routine (version 6.1) --
+ * -- Distributed SuperLU routine (version 7.1.0) --
  * Lawrence Berkeley National Lab, Univ. of California Berkeley.
  * March 15, 2003
  *
  * Modified:
  *     Feburary 7, 2001    use MPI_Isend/MPI_Irecv
  *     October 2, 2001     use MPI_Isend/MPI_Irecv with MPI_Test
- * February 8, 2019  version 6.1.1
+ *     February 8, 2019  version 6.1.1
+ *     October 5, 2021   version 7.1.0  disable a few 'omp simd'
  * </pre>
  */
 
@@ -71,7 +72,7 @@ void zlsum_fmod
  int_t luptr,     /* Starting position in lusup[*].                     */
  int_t *xsup,
  gridinfo_t *grid,
- LocalLU_t *Llu,
+ zLocalLU_t *Llu,
  MPI_Request send_req[], /* input/output */
  SuperLUStat_t *stat
 )
@@ -258,7 +259,7 @@ void zlsum_bmod
  int_t  **Ucb_valptr, /* Vertical linked list pointing to Unzval[].     */
  int_t  *xsup,
  gridinfo_t *grid,
- LocalLU_t *Llu,
+ zLocalLU_t *Llu,
  MPI_Request send_req[], /* input/output */
  SuperLUStat_t *stat
  )
@@ -433,7 +434,7 @@ void zlsum_fmod_inv
  int_t *fmod,     /* Modification count for L-solve.                    */
  int_t *xsup,
  gridinfo_t *grid,
- LocalLU_t *Llu,
+ zLocalLU_t *Llu,
  SuperLUStat_t **stat,
  int_t *leaf_send,
  int_t *nleaf_send,
@@ -539,11 +540,11 @@ void zlsum_fmod_inv
 			for (nn=0;nn<Nchunk;++nn){
 
 #ifdef _OPENMP
-				thread_id1 = omp_get_thread_num ();
+			    thread_id1 = omp_get_thread_num ();
 #else
-				thread_id1 = 0;
+			    thread_id1 = 0;
 #endif
-				rtemp_loc = &rtemp[sizertemp* thread_id1];
+			    rtemp_loc = &rtemp[sizertemp* thread_id1];
 
 				if(nn<remainder){
 					lbstart = nn*(nlb_loc+1);
@@ -565,45 +566,45 @@ void zlsum_fmod_inv
 						nbrow += lsub[lptr1_tmp+1];
 					}
 
-				#ifdef _CRAY
+#ifdef _CRAY
 					CGEMM( ftcs2, ftcs2, &nbrow, &nrhs, &knsupc,
 						  &alpha, &lusup[luptr_tmp1], &nsupr, xk,
 						  &knsupc, &beta, rtemp_loc, &nbrow );
-				#elif defined (USE_VENDOR_BLAS)
+#elif defined (USE_VENDOR_BLAS)
 					zgemm_( "N", "N", &nbrow, &nrhs, &knsupc,
 						   &alpha, &lusup[luptr_tmp1], &nsupr, xk,
 						   &knsupc, &beta, rtemp_loc, &nbrow, 1, 1 );
-				#else
+#else
 					zgemm_( "N", "N", &nbrow, &nrhs, &knsupc,
 						   &alpha, &lusup[luptr_tmp1], &nsupr, xk,
 						   &knsupc, &beta, rtemp_loc, &nbrow );
-				#endif
+#endif
 
 					nbrow_ref=0;
 					for (lb = lbstart; lb < lbend; ++lb){
-						lptr1_tmp = lloc[lb+idx_i];
-						lptr= lptr1_tmp+2;
-						nbrow1 = lsub[lptr1_tmp+1];
-						ik = lsub[lptr1_tmp]; /* Global block number, row-wise. */
-						rel = xsup[ik]; /* Global row index of block ik. */
+					    lptr1_tmp = lloc[lb+idx_i];
+					    lptr= lptr1_tmp+2;
+					    nbrow1 = lsub[lptr1_tmp+1];
+					    ik = lsub[lptr1_tmp]; /* Global block number, row-wise. */
+					    rel = xsup[ik]; /* Global row index of block ik. */
 
-						lk = LBi( ik, grid ); /* Local block number, row-wise. */
+					    lk = LBi( ik, grid ); /* Local block number, row-wise. */
 
-						iknsupc = SuperSize( ik );
-						il = LSUM_BLK( lk );
+					    iknsupc = SuperSize( ik );
+					    il = LSUM_BLK( lk );
 
-						RHS_ITERATE(j)
-							#ifdef _OPENMP
-							#pragma omp simd
-							#endif
-							for (i = 0; i < nbrow1; ++i) {
-								irow = lsub[lptr+i] - rel; /* Relative row. */
-								z_sub(&lsum[il+irow + j*iknsupc+sizelsum*thread_id1],
-									  &lsum[il+irow + j*iknsupc+sizelsum*thread_id1],
-									  &rtemp_loc[nbrow_ref+i + j*nbrow]);
-							}
+					    RHS_ITERATE(j)
+		#ifdef _OPENMP
+		#pragma omp simd
+		#endif
+						for (i = 0; i < nbrow1; ++i) {
+					   	    irow = lsub[lptr+i] - rel; /* Relative row. */
+						    z_sub(&lsum[il+irow + j*iknsupc+sizelsum*thread_id1],
+							  &lsum[il+irow + j*iknsupc+sizelsum*thread_id1],
+							  &rtemp_loc[nbrow_ref+i + j*nbrow]);
+						}
 						nbrow_ref+=nbrow1;
-					}
+					} /* endd for lb ... */
 
 #if ( PROFlevel>=1 )
 					TOC(t2, t1);
@@ -611,160 +612,155 @@ void zlsum_fmod_inv
 #endif
 
 					for (lb=lbstart;lb<lbend;lb++){
-						lk = lloc[lb+idx_n];
+					    lk = lloc[lb+idx_n];
 #ifdef _OPENMP
 #pragma omp atomic capture
 #endif
-						fmod_tmp=--fmod[lk*aln_i];
+					    fmod_tmp=--fmod[lk*aln_i];
 
-						if ( fmod_tmp==0 ) { /* Local accumulation done. */
+					    if ( fmod_tmp==0 ) { /* Local accumulation done. */
 
-							lptr1_tmp = lloc[lb+idx_i];
+						lptr1_tmp = lloc[lb+idx_i];
 
-							ik = lsub[lptr1_tmp]; /* Global block number, row-wise. */
-							lk = LBi( ik, grid ); /* Local block number, row-wise. */
+						ik = lsub[lptr1_tmp]; /* Global block number, row-wise. */
+						lk = LBi( ik, grid ); /* Local block number, row-wise. */
 
-							iknsupc = SuperSize( ik );
-							il = LSUM_BLK( lk );
+						iknsupc = SuperSize( ik );
+						il = LSUM_BLK( lk );
 
-							ikcol = PCOL( ik, grid );
-							p = PNUM( myrow, ikcol, grid );
-							if ( iam != p ) {
-								for (ii=1;ii<num_thread;ii++)
-									#ifdef _OPENMP
-									#pragma omp simd
-									#endif
-									for (jj=0;jj<iknsupc*nrhs;jj++)
-										z_add(&lsum[il + jj ],
-											  &lsum[il + jj ],
-											  &lsum[il + jj + ii*sizelsum]);
+						ikcol = PCOL( ik, grid );
+						p = PNUM( myrow, ikcol, grid );
+						if ( iam != p ) {
+						    for (ii=1;ii<num_thread;ii++)
+						#ifdef _OPENMP
+						#pragma omp simd
+						#endif
+							for (jj=0;jj<iknsupc*nrhs;jj++)
+								z_add(&lsum[il + jj ],
+									  &lsum[il + jj ],
+									  &lsum[il + jj + ii*sizelsum]);
 
 #ifdef _OPENMP
 #pragma omp atomic capture
+#endif
+							nleaf_send_tmp = ++nleaf_send[0];
+							leaf_send[(nleaf_send_tmp-1)*aln_i] = -lk-1;
+							// RdTree_forwardMessageSimple(LRtree_ptr[lk],&lsum[il - LSUM_H ],'z');
+
+						} else { /* Diagonal process: X[i] += lsum[i]. */
+
+#if ( PROFlevel>=1 )
+							TIC(t1);
+#endif
+							for (ii=1;ii<num_thread;ii++)
+						#ifdef _OPENMP
+						#pragma omp simd
+						#endif
+							    for (jj=0;jj<iknsupc*nrhs;jj++)
+								z_add(&lsum[il + jj ],
+									  &lsum[il + jj ],
+									  &lsum[il + jj + ii*sizelsum]);
+
+							ii = X_BLK( lk );
+							RHS_ITERATE(j)
+						#ifdef _OPENMP
+						#pragma omp simd
+						#endif
+							    for (i = 0; i < iknsupc; ++i)
+								z_add(&x[i + ii + j*iknsupc],
+									  &x[i + ii + j*iknsupc],
+									  &lsum[i + il + j*iknsupc] );
+
+							// fmod[lk] = -1; /* Do not solve X[k] in the future. */
+							lk = LBj( ik, grid );/* Local block number, column-wise. */
+							lsub1 = Llu->Lrowind_bc_ptr[lk];
+							lusup1 = Llu->Lnzval_bc_ptr[lk];
+							nsupr1 = lsub1[1];
+
+							if(Llu->inv == 1){
+								Linv = Llu->Linv_bc_ptr[lk];
+
+
+#ifdef _CRAY
+								CGEMM( ftcs2, ftcs2, &iknsupc, &nrhs, &iknsupc,
+										&alpha, Linv, &iknsupc, &x[ii],
+										&iknsupc, &beta, rtemp_loc, &iknsupc );
+#elif defined (USE_VENDOR_BLAS)
+								zgemm_( "N", "N", &iknsupc, &nrhs, &iknsupc,
+										&alpha, Linv, &iknsupc, &x[ii],
+										&iknsupc, &beta, rtemp_loc, &iknsupc, 1, 1 );
+#else
+								zgemm_( "N", "N", &iknsupc, &nrhs, &iknsupc,
+										&alpha, Linv, &iknsupc, &x[ii],
+										&iknsupc, &beta, rtemp_loc, &iknsupc );
+#endif
+							#ifdef _OPENMP
+							#pragma omp simd
+							#endif
+								for (i=0 ; i<iknsupc*nrhs ; i++){
+									z_copy(&x[ii+i],&rtemp_loc[i]);
+								}
+
+							}else{
+#ifdef _CRAY
+								CTRSM(ftcs1, ftcs1, ftcs2, ftcs3, &iknsupc, &nrhs, &alpha,
+											lusup1, &nsupr1, &x[ii], &iknsupc);
+#elif defined (USE_VENDOR_BLAS)
+								ztrsm_("L", "L", "N", "U", &iknsupc, &nrhs, &alpha,
+										lusup1, &nsupr1, &x[ii], &iknsupc, 1, 1, 1, 1);
+#else
+								ztrsm_("L", "L", "N", "U", &iknsupc, &nrhs, &alpha,
+										lusup1, &nsupr1, &x[ii], &iknsupc);
+
+#endif
+							} /* end else */
+
+#if ( PROFlevel>=1 )
+							TOC(t2, t1);
+							stat[thread_id1]->utime[SOL_TRSM] += t2;
+
+#endif
+
+							stat[thread_id1]->ops[SOLVE] += 4 * iknsupc * (iknsupc - 1) * nrhs
+								+ 10 * knsupc * nrhs; /* complex division */
+
+#if ( DEBUGlevel>=2 )
+							printf("(%2d) Solve X[%2d]\n", iam, ik);
+
+#endif
+
+							/*
+							 * Send Xk to process column Pc[k].
+							 */
+
+							if(LBtree_ptr[lk]!=NULL){
+#ifdef _OPENMP
+#pragma omp atomic capture
 #endif
 								nleaf_send_tmp = ++nleaf_send[0];
-								leaf_send[(nleaf_send_tmp-1)*aln_i] = -lk-1;
-								// RdTree_forwardMessageSimple(LRtree_ptr[lk],&lsum[il - LSUM_H ],'z');
+								leaf_send[(nleaf_send_tmp-1)*aln_i] = lk;
+							}
 
-							} else { /* Diagonal process: X[i] += lsum[i]. */
+							/*
+							 * Perform local block modifications.
+							 */
 
-#if ( PROFlevel>=1 )
-								TIC(t1);
-#endif
-								for (ii=1;ii<num_thread;ii++)
-									#ifdef _OPENMP
-									#pragma omp simd
-									#endif
-									for (jj=0;jj<iknsupc*nrhs;jj++)
-										z_add(&lsum[il + jj ],
-											  &lsum[il + jj ],
-											  &lsum[il + jj + ii*sizelsum]);
+// #ifdef _OPENMP
+// #pragma	omp	task firstprivate (Llu,sizelsum,iknsupc,ii,ik,lsub1,x,rtemp,fmod,lsum,stat,nrhs,grid,xsup,recurlevel) private(lptr1,luptr1,nlb1,thread_id1) untied priority(1)
+// #endif
+							{
 
-								ii = X_BLK( lk );
-								RHS_ITERATE(j)
-									#ifdef _OPENMP
-									#pragma omp simd
-									#endif
-									for (i = 0; i < iknsupc; ++i)
-										z_add(&x[i + ii + j*iknsupc],
-											  &x[i + ii + j*iknsupc],
-											  &lsum[i + il + j*iknsupc] );
+								zlsum_fmod_inv(lsum, x, &x[ii], rtemp, nrhs, ik,
+										fmod, xsup,
+										grid, Llu, stat, leaf_send, nleaf_send ,sizelsum,sizertemp,1+recurlevel,maxsuper,thread_id1,num_thread);
+							}
 
-
-								// fmod[lk] = -1; /* Do not solve X[k] in the future. */
-								lk = LBj( ik, grid );/* Local block number, column-wise. */
-								lsub1 = Llu->Lrowind_bc_ptr[lk];
-								lusup1 = Llu->Lnzval_bc_ptr[lk];
-								nsupr1 = lsub1[1];
-
-								if(Llu->inv == 1){
-									Linv = Llu->Linv_bc_ptr[lk];
-
-
-#ifdef _CRAY
-									CGEMM( ftcs2, ftcs2, &iknsupc, &nrhs, &iknsupc,
-											&alpha, Linv, &iknsupc, &x[ii],
-											&iknsupc, &beta, rtemp_loc, &iknsupc );
-#elif defined (USE_VENDOR_BLAS)
-									zgemm_( "N", "N", &iknsupc, &nrhs, &iknsupc,
-											&alpha, Linv, &iknsupc, &x[ii],
-											&iknsupc, &beta, rtemp_loc, &iknsupc, 1, 1 );
-#else
-									zgemm_( "N", "N", &iknsupc, &nrhs, &iknsupc,
-											&alpha, Linv, &iknsupc, &x[ii],
-											&iknsupc, &beta, rtemp_loc, &iknsupc );
-#endif
-									#ifdef _OPENMP
-									#pragma omp simd
-									#endif
-									for (i=0 ; i<iknsupc*nrhs ; i++){
-										z_copy(&x[ii+i],&rtemp_loc[i]);
-									}
-
-								}else{
-#ifdef _CRAY
-									CTRSM(ftcs1, ftcs1, ftcs2, ftcs3, &iknsupc, &nrhs, &alpha,
-											lusup1, &nsupr1, &x[ii], &iknsupc);
-#elif defined (USE_VENDOR_BLAS)
-									ztrsm_("L", "L", "N", "U", &iknsupc, &nrhs, &alpha,
-											lusup1, &nsupr1, &x[ii], &iknsupc, 1, 1, 1, 1);
-#else
-									ztrsm_("L", "L", "N", "U", &iknsupc, &nrhs, &alpha,
-											lusup1, &nsupr1, &x[ii], &iknsupc);
-
-#endif
-								}
-								// for (i=0 ; i<iknsupc*nrhs ; i++){
-								// printf("x_lsum: %f %f\n",x[ii+i].r,x[ii+i].i);
-								// fflush(stdout);
-								// }
-
-#if ( PROFlevel>=1 )
-								TOC(t2, t1);
-								stat[thread_id1]->utime[SOL_TRSM] += t2;
-
-#endif
-
-								stat[thread_id1]->ops[SOLVE] += 4 * iknsupc * (iknsupc - 1) * nrhs
-								+ 10 * knsupc * nrhs; /* complex division */
-
-#if ( DEBUGlevel>=2 )
-								printf("(%2d) Solve X[%2d]\n", iam, ik);
-
-#endif
-
-								/*
-								 * Send Xk to process column Pc[k].
-								 */
-
-								if(LBtree_ptr[lk]!=NULL){
-#ifdef _OPENMP
-#pragma omp atomic capture
-#endif
-									nleaf_send_tmp = ++nleaf_send[0];
-									leaf_send[(nleaf_send_tmp-1)*aln_i] = lk;
-								}
-
-								/*
-								 * Perform local block modifications.
-								 */
-
-								// #ifdef _OPENMP
-								// #pragma	omp	task firstprivate (Llu,sizelsum,iknsupc,ii,ik,lsub1,x,rtemp,fmod,lsum,stat,nrhs,grid,xsup,recurlevel) private(lptr1,luptr1,nlb1,thread_id1) untied priority(1)
-								// #endif
-								{
-
-									zlsum_fmod_inv(lsum, x, &x[ii], rtemp, nrhs, ik,
-											fmod, xsup,
-											grid, Llu, stat, leaf_send, nleaf_send ,sizelsum,sizertemp,1+recurlevel,maxsuper,thread_id1,num_thread);
-								}
-
-								// } /* if frecv[lk] == 0 */
-						} /* if iam == p */
+							// } /* if frecv[lk] == 0 */
+						} /* end if iam == p */
 					} /* if fmod[lk] == 0 */
 				}
 
-			}
+			} /* end tasklook for nn ... */
 		}
 
 		}else{
@@ -806,18 +802,18 @@ void zlsum_fmod_inv
 				il = LSUM_BLK( lk );
 
 				RHS_ITERATE(j)
-					#ifdef _OPENMP
-					#pragma omp simd
-					#endif
-					for (i = 0; i < nbrow1; ++i) {
-						irow = lsub[lptr+i] - rel; /* Relative row. */
+		#ifdef _OPENMP
+		#pragma omp simd
+		#endif
+				    for (i = 0; i < nbrow1; ++i) {
+					irow = lsub[lptr+i] - rel; /* Relative row. */
 
-								z_sub(&lsum[il+irow + j*iknsupc+sizelsum*thread_id],
-									  &lsum[il+irow + j*iknsupc+sizelsum*thread_id],
-									  &rtemp_loc[nbrow_ref+i + j*nbrow]);
-					}
+					z_sub(&lsum[il+irow + j*iknsupc+sizelsum*thread_id],
+						  &lsum[il+irow + j*iknsupc+sizelsum*thread_id],
+						  &rtemp_loc[nbrow_ref+i + j*nbrow]);
+				    }
 				nbrow_ref+=nbrow1;
-			}
+			} /* end for lb ... */
 
 			// TOC(t3, t1);
 
@@ -828,153 +824,142 @@ void zlsum_fmod_inv
 
 			for (lb=0;lb<nlb;lb++){
 				lk = lloc[lb+idx_n];
-
 #ifdef _OPENMP
 #pragma omp atomic capture
 #endif
 				fmod_tmp=--fmod[lk*aln_i];
 
-
 				if ( fmod_tmp==0 ) { /* Local accumulation done. */
 
-					lptr1_tmp = lloc[lb+idx_i];
+				    lptr1_tmp = lloc[lb+idx_i];
 
-					ik = lsub[lptr1_tmp]; /* Global block number, row-wise. */
-					lk = LBi( ik, grid ); /* Local block number, row-wise. */
+				    ik = lsub[lptr1_tmp]; /* Global block number, row-wise. */
+				    lk = LBi( ik, grid ); /* Local block number, row-wise. */
 
-					iknsupc = SuperSize( ik );
-					il = LSUM_BLK( lk );
-					ikcol = PCOL( ik, grid );
-					p = PNUM( myrow, ikcol, grid );
-					if ( iam != p ) {
-						for (ii=1;ii<num_thread;ii++)
-							#ifdef _OPENMP
-							#pragma omp simd
-							#endif
-							for (jj=0;jj<iknsupc*nrhs;jj++)
-								z_add(&lsum[il + jj ],
-									  &lsum[il + jj ],
-									  &lsum[il + jj + ii*sizelsum]);
+				    iknsupc = SuperSize( ik );
+				    il = LSUM_BLK( lk );
+				    ikcol = PCOL( ik, grid );
+				    p = PNUM( myrow, ikcol, grid );
+				    if ( iam != p ) {
+					for (ii=1;ii<num_thread;ii++)
+				#ifdef _OPENMP
+				#pragma omp simd
+				#endif
+				    	    for (jj=0;jj<iknsupc*nrhs;jj++)
+						z_add(&lsum[il + jj ],
+							  &lsum[il + jj ],
+							  &lsum[il + jj + ii*sizelsum]);
 
 #ifdef _OPENMP
 #pragma omp atomic capture
+#endif
+					nleaf_send_tmp = ++nleaf_send[0];
+						leaf_send[(nleaf_send_tmp-1)*aln_i] = -lk-1;
+
+				    } else { /* Diagonal process: X[i] += lsum[i]. */
+
+#if ( PROFlevel>=1 )
+					TIC(t1);
+#endif
+					for (ii=1;ii<num_thread;ii++)
+				#ifdef _OPENMP
+				#pragma omp simd
+				#endif
+					    for (jj=0;jj<iknsupc*nrhs;jj++)
+						z_add(&lsum[il + jj ],
+							  &lsum[il + jj ],
+							  &lsum[il + jj + ii*sizelsum]);
+
+					ii = X_BLK( lk );
+					RHS_ITERATE(j)
+				#ifdef _OPENMP
+				#pragma omp simd
+				#endif
+				 	    for (i = 0; i < iknsupc; ++i)
+						z_add(&x[i + ii + j*iknsupc],
+							  &x[i + ii + j*iknsupc],
+							  &lsum[i + il + j*iknsupc] );
+
+					lk = LBj( ik, grid );/* Local block number, column-wise. */
+					lsub1 = Llu->Lrowind_bc_ptr[lk];
+					lusup1 = Llu->Lnzval_bc_ptr[lk];
+					nsupr1 = lsub1[1];
+
+					if(Llu->inv == 1){
+					    Linv = Llu->Linv_bc_ptr[lk];
+#ifdef _CRAY
+					    CGEMM( ftcs2, ftcs2, &iknsupc, &nrhs, &iknsupc,
+							&alpha, Linv, &iknsupc, &x[ii],
+							&iknsupc, &beta, rtemp_loc, &iknsupc );
+#elif defined (USE_VENDOR_BLAS)
+					    zgemm_( "N", "N", &iknsupc, &nrhs, &iknsupc,
+							&alpha, Linv, &iknsupc, &x[ii],
+							&iknsupc, &beta, rtemp_loc, &iknsupc, 1, 1 );
+#else
+					    zgemm_( "N", "N", &iknsupc, &nrhs, &iknsupc,
+							&alpha, Linv, &iknsupc, &x[ii],
+							&iknsupc, &beta, rtemp_loc, &iknsupc );
+#endif
+					#ifdef _OPENMP
+					#pragma omp simd
+					#endif
+					    for (i=0 ; i<iknsupc*nrhs ; i++){
+						z_copy(&x[ii+i],&rtemp_loc[i]);
+					    }
+					}else{
+#ifdef _CRAY
+					    CTRSM(ftcs1, ftcs1, ftcs2, ftcs3, &iknsupc, &nrhs, &alpha,
+							lusup1, &nsupr1, &x[ii], &iknsupc);
+#elif defined (USE_VENDOR_BLAS)
+					    ztrsm_("L", "L", "N", "U", &iknsupc, &nrhs, &alpha,
+							lusup1, &nsupr1, &x[ii], &iknsupc, 1, 1, 1, 1);
+#else
+					    ztrsm_("L", "L", "N", "U", &iknsupc, &nrhs, &alpha,
+									lusup1, &nsupr1, &x[ii], &iknsupc);
+#endif
+					} /* end else */
+
+#if ( PROFlevel>=1 )
+					TOC(t2, t1);
+					stat[thread_id]->utime[SOL_TRSM] += t2;
+#endif
+
+					stat[thread_id]->ops[SOLVE] += 4 * iknsupc * (iknsupc - 1) * nrhs
+						+ 10 * knsupc * nrhs; /* complex division */
+
+#if ( DEBUGlevel>=2 )
+					printf("(%2d) Solve X[%2d]\n", iam, ik);
+#endif
+
+					/*
+					 * Send Xk to process column Pc[k].
+					 */
+
+					if(LBtree_ptr[lk]!=NULL){
+#ifdef _OPENMP
+#pragma omp atomic capture
 #endif
 						nleaf_send_tmp = ++nleaf_send[0];
-						leaf_send[(nleaf_send_tmp-1)*aln_i] = -lk-1;
+						// printf("nleaf_send_tmp %5d lk %5d\n",nleaf_send_tmp);
+						leaf_send[(nleaf_send_tmp-1)*aln_i] = lk;
+						// BcTree_forwardMessageSimple(LBtree_ptr[lk],&x[ii - XK_H],'z');
+					}
 
-					} else { /* Diagonal process: X[i] += lsum[i]. */
+					/*
+					 * Perform local block modifications.
+					 */
 
-#if ( PROFlevel>=1 )
-						TIC(t1);
-#endif
-						for (ii=1;ii<num_thread;ii++)
-							#ifdef _OPENMP
-							#pragma omp simd
-							#endif
-							for (jj=0;jj<iknsupc*nrhs;jj++)
-								z_add(&lsum[il + jj ],
-									  &lsum[il + jj ],
-									  &lsum[il + jj + ii*sizelsum]);
-
-						ii = X_BLK( lk );
-						RHS_ITERATE(j)
-							#ifdef _OPENMP
-							#pragma omp simd
-							#endif
-							for (i = 0; i < iknsupc; ++i)
-								z_add(&x[i + ii + j*iknsupc],
-									  &x[i + ii + j*iknsupc],
-									  &lsum[i + il + j*iknsupc] );
-
-
-						lk = LBj( ik, grid );/* Local block number, column-wise. */
-						lsub1 = Llu->Lrowind_bc_ptr[lk];
-						lusup1 = Llu->Lnzval_bc_ptr[lk];
-						nsupr1 = lsub1[1];
-
-						if(Llu->inv == 1){
-							Linv = Llu->Linv_bc_ptr[lk];
-#ifdef _CRAY
-							CGEMM( ftcs2, ftcs2, &iknsupc, &nrhs, &iknsupc,
-									&alpha, Linv, &iknsupc, &x[ii],
-									&iknsupc, &beta, rtemp_loc, &iknsupc );
-#elif defined (USE_VENDOR_BLAS)
-							zgemm_( "N", "N", &iknsupc, &nrhs, &iknsupc,
-									&alpha, Linv, &iknsupc, &x[ii],
-									&iknsupc, &beta, rtemp_loc, &iknsupc, 1, 1 );
-#else
-							zgemm_( "N", "N", &iknsupc, &nrhs, &iknsupc,
-									&alpha, Linv, &iknsupc, &x[ii],
-									&iknsupc, &beta, rtemp_loc, &iknsupc );
-#endif
-							#ifdef _OPENMP
-							#pragma omp simd
-							#endif
-							for (i=0 ; i<iknsupc*nrhs ; i++){
-								z_copy(&x[ii+i],&rtemp_loc[i]);
-							}
-						}else{
-#ifdef _CRAY
-							CTRSM(ftcs1, ftcs1, ftcs2, ftcs3, &iknsupc, &nrhs, &alpha,
-									lusup1, &nsupr1, &x[ii], &iknsupc);
-#elif defined (USE_VENDOR_BLAS)
-							ztrsm_("L", "L", "N", "U", &iknsupc, &nrhs, &alpha,
-									lusup1, &nsupr1, &x[ii], &iknsupc, 1, 1, 1, 1);
-#else
-							ztrsm_("L", "L", "N", "U", &iknsupc, &nrhs, &alpha,
-									lusup1, &nsupr1, &x[ii], &iknsupc);
-#endif
-						}
-
-							// for (i=0 ; i<iknsupc*nrhs ; i++){
-							// printf("x_lsum: %f %f\n",x[ii+i].r,x[ii+i].i);
-							// fflush(stdout);
-							// }
-
-
-#if ( PROFlevel>=1 )
-						TOC(t2, t1);
-						stat[thread_id]->utime[SOL_TRSM] += t2;
-#endif
-
-						stat[thread_id]->ops[SOLVE] += 4 * iknsupc * (iknsupc - 1) * nrhs
-						+ 10 * knsupc * nrhs; /* complex division */
-
-#if ( DEBUGlevel>=2 )
-						printf("(%2d) Solve X[%2d]\n", iam, ik);
-#endif
-
-						/*
-						 * Send Xk to process column Pc[k].
-						 */
-
-						if(LBtree_ptr[lk]!=NULL){
-
-#ifdef _OPENMP
-#pragma omp atomic capture
-#endif
-							nleaf_send_tmp = ++nleaf_send[0];
-							// printf("nleaf_send_tmp %5d lk %5d\n",nleaf_send_tmp);
-							leaf_send[(nleaf_send_tmp-1)*aln_i] = lk;
-							// BcTree_forwardMessageSimple(LBtree_ptr[lk],&x[ii - XK_H],'z');
-						}
-
-						/*
-						 * Perform local block modifications.
-						 */
-
-						// #ifdef _OPENMP
-						// #pragma	omp	task firstprivate (Llu,sizelsum,iknsupc,ii,ik,lsub1,x,rtemp,fmod,lsum,stat,nrhs,grid,xsup,recurlevel) private(lptr1,luptr1,nlb1) untied priority(1)
-						// #endif
-
-						{
-							zlsum_fmod_inv(lsum, x, &x[ii], rtemp, nrhs, ik,
-									fmod, xsup,
-									grid, Llu, stat, leaf_send, nleaf_send ,sizelsum,sizertemp,1+recurlevel,maxsuper,thread_id,num_thread);
-						}
+// #ifdef _OPENMP
+// #pragma	omp	task firstprivate (Llu,sizelsum,iknsupc,ii,ik,lsub1,x,rtemp,fmod,lsum,stat,nrhs,grid,xsup,recurlevel) private(lptr1,luptr1,nlb1) untied priority(1)
+// #endif
+					{
+						zlsum_fmod_inv(lsum, x, &x[ii], rtemp, nrhs, ik,
+							fmod, xsup,
+							grid, Llu, stat, leaf_send, nleaf_send ,sizelsum,sizertemp,1+recurlevel,maxsuper,thread_id,num_thread);
+					}
 
 						// } /* if frecv[lk] == 0 */
-				} /* if iam == p */
+				} /* end else iam == p */
 			} /* if fmod[lk] == 0 */
 		}
 		// }
@@ -983,7 +968,6 @@ void zlsum_fmod_inv
 	stat[thread_id]->ops[SOLVE] += 8 * m * nrhs * knsupc;
 
 
-
 } /* if nlb>0*/
 } /* zLSUM_FMOD_INV */
 
@@ -1010,7 +994,7 @@ void zlsum_fmod_inv_master
  int_t nlb,       /* Number of L blocks.                                */
  int_t *xsup,
  gridinfo_t *grid,
- LocalLU_t *Llu,
+ zLocalLU_t *Llu,
  SuperLUStat_t **stat,
  int_t sizelsum,
  int_t sizertemp,
@@ -1167,9 +1151,9 @@ void zlsum_fmod_inv_master
 						il = LSUM_BLK( lk );
 
 						RHS_ITERATE(j)
-							#ifdef _OPENMP
-								#pragma omp simd lastprivate(irow)
-							#endif
+					#ifdef _OPENMP
+					#pragma omp simd lastprivate(irow)
+					#endif
 							for (i = 0; i < nbrow1; ++i) {
 								irow = lsub[lptr+i] - rel; /* Relative row. */
 								z_sub(&lsum[il+irow + j*iknsupc],
@@ -1177,14 +1161,15 @@ void zlsum_fmod_inv_master
 									  &rtemp_loc[nbrow_ref+i + j*nbrow]);
 							}
 						nbrow_ref+=nbrow1;
-					}
+					} /* end for lb ... */
 
 #if ( PROFlevel>=1 )
 					TOC(t2, t1);
 					stat[thread_id1]->utime[SOL_GEMM] += t2;
 #endif
-			}
-		}
+			} /* end if (lbstart<lbend) ... */
+
+		} /* end taskloop for nn = ... */
 
 		}else{
 
@@ -1225,24 +1210,24 @@ void zlsum_fmod_inv_master
 				il = LSUM_BLK( lk );
 
 				RHS_ITERATE(j)
-					#ifdef _OPENMP
-						#pragma omp simd lastprivate(irow)
-					#endif
+			#ifdef _OPENMP
+			#pragma omp simd lastprivate(irow)
+			#endif
 					for (i = 0; i < nbrow1; ++i) {
 						irow = lsub[lptr+i] - rel; /* Relative row. */
 
-								z_sub(&lsum[il+irow + j*iknsupc+sizelsum*thread_id],
-									  &lsum[il+irow + j*iknsupc+sizelsum*thread_id],
-									  &rtemp_loc[nbrow_ref+i + j*nbrow]);
+						z_sub(&lsum[il+irow + j*iknsupc+sizelsum*thread_id],
+							  &lsum[il+irow + j*iknsupc+sizelsum*thread_id],
+							  &rtemp_loc[nbrow_ref+i + j*nbrow]);
 					}
 				nbrow_ref+=nbrow1;
-			}
+			} /* end for lb ... */
 #if ( PROFlevel>=1 )
 			TOC(t2, t1);
 			stat[thread_id]->utime[SOL_GEMM] += t2;
 #endif
-		}
-			// TOC(t3, t1);
+		} /* end else ... */
+		// TOC(t3, t1);
 		rtemp_loc = &rtemp[sizertemp* thread_id];
 
 		for (lb=0;lb<nlb;lb++){
@@ -1277,9 +1262,9 @@ void zlsum_fmod_inv_master
 
 					for (ii=1;ii<num_thread;ii++)
 						// if(ii!=thread_id)
-						#ifdef _OPENMP
-							#pragma omp simd
-						#endif
+				#ifdef _OPENMP
+				#pragma omp simd
+				#endif
 						for (jj=0;jj<iknsupc*nrhs;jj++)
 							z_add(&lsum[il + jj ],
 								  &lsum[il + jj ],
@@ -1291,18 +1276,15 @@ void zlsum_fmod_inv_master
 
 				} else { /* Diagonal process: X[i] += lsum[i]. */
 
-
-
 					// if ( frecv[lk]==0 ) { /* Becomes a leaf node. */
-
 #if ( PROFlevel>=1 )
 					TIC(t1);
 #endif
 					for (ii=1;ii<num_thread;ii++)
 						// if(ii!=thread_id)
-						#ifdef _OPENMP
-							#pragma omp simd
-						#endif
+				#ifdef _OPENMP
+				#pragma omp simd
+				#endif
 						for (jj=0;jj<iknsupc*nrhs;jj++)
 							z_add(&lsum[il + jj ],
 								  &lsum[il + jj ],
@@ -1311,9 +1293,9 @@ void zlsum_fmod_inv_master
 					ii = X_BLK( lk );
 					// for (jj=0;jj<num_thread;jj++)
 					RHS_ITERATE(j)
-						#ifdef _OPENMP
-							#pragma omp simd
-						#endif
+				#ifdef _OPENMP
+				#pragma omp simd
+				#endif
 						for (i = 0; i < iknsupc; ++i)
 							z_add(&x[i + ii + j*iknsupc],
 								  &x[i + ii + j*iknsupc],
@@ -1340,11 +1322,11 @@ void zlsum_fmod_inv_master
 								&alpha, Linv, &iknsupc, &x[ii],
 								&iknsupc, &beta, rtemp_loc, &iknsupc );
 #endif
-						#ifdef _OPENMP
-							#pragma omp simd
-						#endif
+				#ifdef _OPENMP
+				#pragma omp simd
+				#endif
 						for (i=0 ; i<iknsupc*nrhs ; i++){
-										z_copy(&x[ii+i],&rtemp_loc[i]);
+							z_copy(&x[ii+i],&rtemp_loc[i]);
 						}
 					}else{
 #ifdef _CRAY
@@ -1358,19 +1340,14 @@ void zlsum_fmod_inv_master
 								lusup1, &nsupr1, &x[ii], &iknsupc);
 #endif
 					}
-					// for (i=0 ; i<iknsupc*nrhs ; i++){
-					// printf("x_usum: %f %f\n",x[ii+i].r,x[ii+i].i);
-					// fflush(stdout);
-					// }
 
 #if ( PROFlevel>=1 )
 					TOC(t2, t1);
 					stat[thread_id]->utime[SOL_TRSM] += t2;
-
 #endif
 
 					stat[thread_id]->ops[SOLVE] += 4 * iknsupc * (iknsupc - 1) * nrhs
-					+ 10 * knsupc * nrhs; /* complex division */
+								    + 10 * knsupc * nrhs; /* complex division */
 
 #if ( DEBUGlevel>=2 )
 					printf("(%2d) Solve X[%2d]\n", iam, ik);
@@ -1387,13 +1364,12 @@ void zlsum_fmod_inv_master
 					 * Perform local block modifications.
 					 */
 
-					// #ifdef _OPENMP
-					// #pragma	omp	task firstprivate (Llu,sizelsum,iknsupc,ii,ik,lsub1,x,rtemp,fmod,lsum,stat,nrhs,grid,xsup,recurlevel) private(lptr1,luptr1,nlb1,thread_id1) untied priority(1)
-					// #endif
+// #ifdef _OPENMP
+// #pragma	omp	task firstprivate (Llu,sizelsum,iknsupc,ii,ik,lsub1,x,rtemp,fmod,lsum,stat,nrhs,grid,xsup,recurlevel) private(lptr1,luptr1,nlb1,thread_id1) untied priority(1)
+// #endif
 					{
 						nlb1 = lsub1[0] - 1;
 
-
 						zlsum_fmod_inv_master(lsum, x, &x[ii], rtemp, nrhs, iknsupc, ik,
 								fmod, nlb1, xsup,
 								grid, Llu, stat,sizelsum,sizertemp,1+recurlevel,maxsuper,thread_id,num_thread);
@@ -1405,8 +1381,8 @@ void zlsum_fmod_inv_master
 		}
 		// }
 		stat[thread_id]->ops[SOLVE] += 8 * m * nrhs * knsupc;
-	} /* if nlb>0*/
-} /* zLSUM_FMOD_INV */
+	} /* end if nlb>0*/
+} /* end zlsum_fmod_inv_master */
 
 
 
@@ -1426,7 +1402,7 @@ void zlsum_bmod_inv
  int_t  **Ucb_valptr, /* Vertical linked list pointing to Unzval[].     */
  int_t  *xsup,
  gridinfo_t *grid,
- LocalLU_t *Llu,
+ zLocalLU_t *Llu,
  SuperLUStat_t **stat,
  int_t* root_send,
  int_t* nroot_send,
@@ -1466,7 +1442,7 @@ void zlsum_bmod_inv
 	float msg_vol = 0, msg_cnt = 0;
 	int_t Nchunk, nub_loc,remainder,nn,lbstart,lbend;
 	int_t iword = sizeof(int_t);
-	int_t dword = sizeof (double);
+	int_t dword = sizeof(double);
 	int_t aln_d,aln_i;
 	aln_d = ceil(CACHELINE/(double)dword);
 	aln_i = ceil(CACHELINE/(double)iword);
@@ -1530,9 +1506,9 @@ void zlsum_bmod_inv
 						fnz = usub[i + jj];
 						if ( fnz < iklrow ) { /* Nonzero segment. */
 							/* AXPY */
-							#ifdef _OPENMP
-							#pragma omp simd
-							#endif
+//#ifdef _OPENMP  
+//#pragma omp simd // In complex case, this SIMD loop has 2 instructions, the compiler may generate incoreect code, so need to disable this omp simd
+//#endif
 							for (irow = fnz; irow < iklrow; ++irow)
 								{
 								zz_mult(&temp, &uval[uptr], &y[jj]);
@@ -1543,7 +1519,7 @@ void zlsum_bmod_inv
 								stat[thread_id1]->ops[SOLVE] += 8 * (iklrow - fnz);
 
 						}
-					} /* for jj ... */
+					} /* end for jj ... */
 				}
 
 #if ( PROFlevel>=1 )
@@ -1551,7 +1527,6 @@ void zlsum_bmod_inv
 				stat[thread_id1]->utime[SOL_GEMM] += t2;
 #endif
 
-
 		#ifdef _OPENMP
 		#pragma omp atomic capture
 		#endif
@@ -1563,9 +1538,9 @@ void zlsum_bmod_inv
 					if ( iam != p ) {
 						for (ii=1;ii<num_thread;ii++)
 							// if(ii!=thread_id1)
-							#ifdef _OPENMP
-							#pragma omp simd
-							#endif
+				#ifdef _OPENMP
+				#pragma omp simd
+				#endif
 							for (jj=0;jj<iknsupc*nrhs;jj++)
 								z_add(&lsum[il + jj ],
 									  &lsum[il + jj ],
@@ -1587,12 +1562,11 @@ void zlsum_bmod_inv
 #if ( PROFlevel>=1 )
 						TIC(t1);
 #endif
-
 						for (ii=1;ii<num_thread;ii++)
 							// if(ii!=thread_id1)
-							#ifdef _OPENMP
-							#pragma omp simd
-							#endif
+				#ifdef _OPENMP
+				#pragma omp simd
+				#endif
 							for (jj=0;jj<iknsupc*nrhs;jj++)
 								z_add(&lsum[il + jj ],
 									  &lsum[il + jj ],
@@ -1602,11 +1576,11 @@ void zlsum_bmod_inv
 						dest = &x[ii];
 
 						RHS_ITERATE(j)
-							#ifdef _OPENMP
-							#pragma omp simd
-							#endif
+				#ifdef _OPENMP
+				#pragma omp simd
+				#endif
 							for (i = 0; i < iknsupc; ++i)
-								z_add(&dest[i + j*iknsupc],
+							    z_add(&dest[i + j*iknsupc],
 									  &dest[i + j*iknsupc],
 									  &lsum[i + il + j*iknsupc]);
 
@@ -1650,10 +1624,6 @@ void zlsum_bmod_inv
 										lusup, &nsupr, &x[ii], &iknsupc);
 		#endif
 							}
-								// for (i=0 ; i<iknsupc*nrhs ; i++){
-								// printf("x_usum: %f %f\n",x[ii+i].r,x[ii+i].i);
-								// fflush(stdout);
-								// }
 
 		#if ( PROFlevel>=1 )
 							TOC(t2, t1);
@@ -1678,8 +1648,8 @@ void zlsum_bmod_inv
 #ifdef _OPENMP
 #pragma omp atomic capture
 #endif
-							nroot_send_tmp = ++nroot_send[0];
-							root_send[(nroot_send_tmp-1)*aln_i] = lk1;
+							    nroot_send_tmp = ++nroot_send[0];
+							    root_send[(nroot_send_tmp-1)*aln_i] = lk1;
 							// BcTree_forwardMessageSimple(UBtree_ptr[lk1],&x[ii - XK_H],'z');
 							}
 
@@ -1687,9 +1657,9 @@ void zlsum_bmod_inv
 							 * Perform local block modifications.
 							 */
 							if ( Urbs[lk1] ){
-								// #ifdef _OPENMP
-								// #pragma	omp	task firstprivate (Ucb_indptr,Ucb_valptr,Llu,sizelsum,ii,gik,x,rtemp,bmod,Urbs,lsum,stat,nrhs,grid,xsup) untied
-								// #endif
+// #ifdef _OPENMP
+// #pragma	omp	task firstprivate (Ucb_indptr,Ucb_valptr,Llu,sizelsum,ii,gik,x,rtemp,bmod,Urbs,lsum,stat,nrhs,grid,xsup) untied
+// #endif
 								{
 								zlsum_bmod_inv(lsum, x, &x[ii], rtemp, nrhs, gik, bmod, Urbs,
 										Ucb_indptr, Ucb_valptr, xsup, grid, Llu,
@@ -1699,8 +1669,8 @@ void zlsum_bmod_inv
 						// } /* if brecv[ik] == 0 */
 					}
 				} /* if bmod[ik] == 0 */
-			}
-		}
+			} /* end for ub = ... */
+		} /* end for taskloop nn = ... */
 
 	} else {
 
@@ -1729,18 +1699,18 @@ void zlsum_bmod_inv
 					fnz = usub[i + jj];
 					if ( fnz < iklrow ) { /* Nonzero segment. */
 						/* AXPY */
-						#ifdef _OPENMP
-						#pragma omp simd
-						#endif
+//#ifdef _OPENMP
+//#pragma omp simd // In complex case, this SIMD loop has 2 instructions, the compiler may generate incoreect code, so need to disable this omp simd
+//#endif
 						for (irow = fnz; irow < iklrow; ++irow)
 
-								{
+							{
 								zz_mult(&temp, &uval[uptr], &y[jj]);
 								z_sub(&dest[irow - ikfrow], &dest[irow - ikfrow],
 									  &temp);
 								++uptr;
-								}
-								stat[thread_id]->ops[SOLVE] += 8 * (iklrow - fnz);
+							}
+							stat[thread_id]->ops[SOLVE] += 8 * (iklrow - fnz);
 					}
 				} /* for jj ... */
 			}
@@ -1761,9 +1731,9 @@ void zlsum_bmod_inv
 				if ( iam != p ) {
 					for (ii=1;ii<num_thread;ii++)
 						// if(ii!=thread_id)
-						#ifdef _OPENMP
-						#pragma omp simd
-						#endif
+			#ifdef _OPENMP
+			#pragma omp simd
+			#endif
 						for (jj=0;jj<iknsupc*nrhs;jj++)
 							z_add(&lsum[il + jj ],
 								  &lsum[il + jj ],
@@ -1787,9 +1757,9 @@ void zlsum_bmod_inv
 
 					for (ii=1;ii<num_thread;ii++)
 						// if(ii!=thread_id)
-						#ifdef _OPENMP
-						#pragma omp simd
-						#endif
+			#ifdef _OPENMP
+			#pragma omp simd
+			#endif
 						for (jj=0;jj<iknsupc*nrhs;jj++)
 								z_add(&lsum[il + jj ],
 									  &lsum[il + jj ],
@@ -1799,9 +1769,9 @@ void zlsum_bmod_inv
 					dest = &x[ii];
 
 					RHS_ITERATE(j)
-						#ifdef _OPENMP
-						#pragma omp simd
-						#endif
+			#ifdef _OPENMP
+			#pragma omp simd
+			#endif
 						for (i = 0; i < iknsupc; ++i)
 							z_add(&dest[i + j*iknsupc],
 								  &dest[i + j*iknsupc],
@@ -1829,9 +1799,9 @@ void zlsum_bmod_inv
 									&alpha, Uinv, &iknsupc, &x[ii],
 									&iknsupc, &beta, rtemp_loc, &iknsupc );
 	#endif
-							#ifdef _OPENMP
-							#pragma omp simd
-							#endif
+				#ifdef _OPENMP
+				#pragma omp simd
+				#endif
 							for (i=0 ; i<iknsupc*nrhs ; i++){
 								z_copy(&x[ii+i],&rtemp_loc[i]);
 							}
@@ -1880,25 +1850,25 @@ void zlsum_bmod_inv
 						 */
 						if ( Urbs[lk1] )
 
-							// if(Urbs[lk1]>16){
-							// #ifdef _OPENMP
-							// #pragma	omp	task firstprivate (Ucb_indptr,Ucb_valptr,Llu,sizelsum,ii,gik,x,rtemp,bmod,Urbs,lsum,stat,nrhs,grid,xsup) untied
-							// #endif
+// if(Urbs[lk1]>16){
+// #ifdef _OPENMP
+// #pragma omp	task firstprivate (Ucb_indptr,Ucb_valptr,Llu,sizelsum,ii,gik,x,rtemp,bmod,Urbs,lsum,stat,nrhs,grid,xsup) untied
+// #endif
 							// 	zlsum_bmod_inv(lsum, x, &x[ii], rtemp, nrhs, gik, bmod, Urbs,
 									//	Ucb_indptr, Ucb_valptr, xsup, grid, Llu,
 									//	stat, root_send, nroot_send, sizelsum,sizertemp);
 							//}else{
-								zlsum_bmod_inv(lsum, x, &x[ii], rtemp, nrhs, gik, bmod, Urbs,
-										Ucb_indptr, Ucb_valptr, xsup, grid, Llu,
-										stat, root_send, nroot_send, sizelsum,sizertemp,thread_id,num_thread);
+							zlsum_bmod_inv(lsum, x, &x[ii], rtemp, nrhs, gik, bmod, Urbs,
+								Ucb_indptr, Ucb_valptr, xsup, grid, Llu,
+								stat, root_send, nroot_send, sizelsum,sizertemp,thread_id,num_thread);
 							//}
 
 					// } /* if brecv[ik] == 0 */
 				}
 			} /* if bmod[ik] == 0 */
 
-		} /* for ub ... */
-	}
+		} /* end for ub ... */
+	} /* end else ... */
 
 } /* zlSUM_BMOD_inv */
 
@@ -1920,7 +1890,7 @@ void zlsum_bmod_inv_master
  int_t  **Ucb_valptr, /* Vertical linked list pointing to Unzval[].     */
  int_t  *xsup,
  gridinfo_t *grid,
- LocalLU_t *Llu,
+ zLocalLU_t *Llu,
  SuperLUStat_t **stat,
  int_t sizelsum,
  int_t sizertemp,
@@ -2025,9 +1995,9 @@ void zlsum_bmod_inv_master
 						fnz = usub[i + jj];
 						if ( fnz < iklrow ) { /* Nonzero segment. */
 							/* AXPY */
-							#ifdef _OPENMP
-							#pragma omp simd
-							#endif
+//#ifdef _OPENMP
+//#pragma omp simd // In complex case, this SIMD loop has 2 instructions, the compiler may generate incoreect code, so need to disable this omp simd
+//#endif
 							for (irow = fnz; irow < iklrow; ++irow)
 								{
 								zz_mult(&temp, &uval[uptr], &y[jj]);
@@ -2072,9 +2042,9 @@ void zlsum_bmod_inv_master
 					fnz = usub[i + jj];
 					if ( fnz < iklrow ) { /* Nonzero segment. */
 						/* AXPY */
-						#ifdef _OPENMP
-						#pragma omp simd
-						#endif
+//#ifdef _OPENMP
+//#pragma omp simd // In complex case, this SIMD loop has 2 instructions, the compiler may generate incoreect code, so need to disable this omp simd
+//#endif
 						for (irow = fnz; irow < iklrow; ++irow)
 							{
 							zz_mult(&temp, &uval[uptr], &y[jj]);
@@ -2113,9 +2083,9 @@ void zlsum_bmod_inv_master
 			if ( iam != p ) {
 				for (ii=1;ii<num_thread;ii++)
 					// if(ii!=thread_id)
-					#ifdef _OPENMP
-					#pragma omp simd
-					#endif
+		#ifdef _OPENMP
+		#pragma omp simd
+		#endif
 					for (jj=0;jj<iknsupc*nrhs;jj++)
 						z_add(&lsum[il + jj ],
 							  &lsum[il + jj ],
@@ -2133,9 +2103,9 @@ void zlsum_bmod_inv_master
 #endif
 				for (ii=1;ii<num_thread;ii++)
 					// if(ii!=thread_id)
-					#ifdef _OPENMP
-					#pragma omp simd
-					#endif
+		#ifdef _OPENMP
+		#pragma omp simd
+		#endif
 					for (jj=0;jj<iknsupc*nrhs;jj++)
 						z_add(&lsum[il + jj ],
 							  &lsum[il + jj ],
@@ -2145,9 +2115,9 @@ void zlsum_bmod_inv_master
 				dest = &x[ii];
 
 				RHS_ITERATE(j)
-					#ifdef _OPENMP
-					#pragma omp simd
-					#endif
+		#ifdef _OPENMP
+		#pragma omp simd
+		#endif
 					for (i = 0; i < iknsupc; ++i)
 						z_add(&dest[i + j*iknsupc],
 							  &dest[i + j*iknsupc],
@@ -2175,9 +2145,9 @@ void zlsum_bmod_inv_master
 								&alpha, Uinv, &iknsupc, &x[ii],
 								&iknsupc, &beta, rtemp_loc, &iknsupc );
 #endif
-						#ifdef _OPENMP
-						#pragma omp simd
-						#endif
+			#ifdef _OPENMP
+			#pragma omp simd
+			#endif
 						for (i=0 ; i<iknsupc*nrhs ; i++){
 							z_copy(&x[ii+i],&rtemp_loc[i]);
 						}
@@ -2220,9 +2190,9 @@ void zlsum_bmod_inv_master
 					 * Perform local block modifications.
 					 */
 					if ( Urbs[lk1] ){
-						// #ifdef _OPENMP
-						// #pragma	omp	task firstprivate (Ucb_indptr,Ucb_valptr,Llu,sizelsum,ii,gik,x,rtemp,bmod,Urbs,lsum,stat,nrhs,grid,xsup) untied
-						// #endif
+// #ifdef _OPENMP
+// #pragma	omp	task firstprivate (Ucb_indptr,Ucb_valptr,Llu,sizelsum,ii,gik,x,rtemp,bmod,Urbs,lsum,stat,nrhs,grid,xsup) untied
+// #endif
 						{
 						zlsum_bmod_inv_master(lsum, x, &x[ii], rtemp, nrhs, gik, bmod, Urbs,
 								Ucb_indptr, Ucb_valptr, xsup, grid, Llu,
