@@ -52,7 +52,7 @@ parse_command_line(int argc, char *argv[], int *nprow, int *npcol,
 extern int
 pdcompute_resid(int m, int n, int nrhs, SuperMatrix *A,
 		double *x, int ldx, double *b, int ldb,
-		gridinfo_t *grid, SOLVEstruct_t *SOLVEstruct, double *resid);
+		gridinfo_t *grid, dSOLVEstruct_t *SOLVEstruct, double *resid);
 
 /*! \brief Copy matrix A into matrix B, in distributed compressed row format. */
 void
@@ -110,9 +110,9 @@ int main(int argc, char *argv[])
     SuperLUStat_t stat;
     SuperMatrix A, Asave;
     NRformat_loc *Astore;
-    ScalePermstruct_t ScalePermstruct;
-    LUstruct_t LUstruct;
-    SOLVEstruct_t SOLVEstruct;
+    dScalePermstruct_t ScalePermstruct;
+    dLUstruct_t LUstruct;
+    dSOLVEstruct_t SOLVEstruct;
     gridinfo_t grid;
     double   *nzval_save;
     int_t    *colind_save, *rowptr_save;
@@ -245,8 +245,8 @@ int main(int argc, char *argv[])
 		        dCopy_CompRowLoc_NoAllocation(&Asave, &A);
 
 		        /* Initialize ScalePermstruct and LUstruct. */
-		        ScalePermstructInit(m, n, &ScalePermstruct);
-		        LUstructInit(n, &LUstruct);
+		        dScalePermstructInit(m, n, &ScalePermstruct);
+		        dLUstructInit(n, &LUstruct);
 
 		        if ( prefact ) {
 
@@ -319,7 +319,7 @@ int main(int argc, char *argv[])
 			    options.Fact = fact;
 			    if ( fact == SamePattern ) {
 			        // {L,U} not re-used in subsequent call to PDGSSVX.
-			        Destroy_LU(n, &grid, &LUstruct);
+			        dDestroy_LU(n, &grid, &LUstruct);
 			    } else if (fact == SamePattern_SameRowPerm) {
 			        // {L,U} structure is re-used in subsequent call to PDGSSVX.
 				dZeroLblocks(iam, n, &grid, &LUstruct);
@@ -337,7 +337,7 @@ int main(int argc, char *argv[])
 			    if (fact == SamePattern_SameRowPerm && iam == 0) {
                                 /* Perturb the 1st diagonal of the matrix 
                                    to larger value, so to have a different A. */
-                                ((double *) Astore->nzval)[0] += 1.0e-8;
+                                ((double *) Astore->nzval)[0] += 1.0e-12; //1.0e-8;
                              }
 
 		        } 
@@ -357,7 +357,7 @@ int main(int argc, char *argv[])
 		        PStatFree(&stat);
 #if 0
 		        pdinf_norm_error(iam, ((NRformat_loc *)A.Store)->m_loc,
-				     nrhs, b, ldb, xtrue, ldx, &grid);
+				     nrhs, b, ldb, xtrue, ldx, grid.comm);
 #endif
 		        if ( info ) {
 			    printf(FMT3, "pdgssvx",info,izero,n,nrhs,imat,nfail);
@@ -375,7 +375,7 @@ int main(int argc, char *argv[])
 			    dgst04(n, nrhs, solx, ldx, xact, ldx, rcond,
 					  &result[2]);
 			    pdinf_norm_error(iam, ((NRformat_loc *)A.Store)->m_loc,
-					 nrhs, b, ldb, xtrue, ldx, &grid);
+					 nrhs, b, ldb, xtrue, ldx, grid.comm);
 #endif
 
 			    /* Print information about the tests that did
@@ -400,9 +400,9 @@ int main(int argc, char *argv[])
 			    SUPERLU_FREE(C);
 			    ScalePermstruct.DiagScale = NOEQUIL; /* Avoid free R/C again. */
 		        }
-		        ScalePermstructFree(&ScalePermstruct);
-		        Destroy_LU(n, &grid, &LUstruct);
-		        LUstructFree(&LUstruct);
+		        dScalePermstructFree(&ScalePermstruct);
+		        dDestroy_LU(n, &grid, &LUstruct);
+		        dLUstructFree(&LUstruct);
 		        if ( options.SolveInitialized ) {
 			    dSolveFinalize(&options, &SOLVEstruct);
 		        }
@@ -421,7 +421,7 @@ int main(int argc, char *argv[])
 	   ------------------------------------------------------------*/
 	Destroy_CompRowLoc_Matrix_dist(&A);
 	Destroy_CompRowLoc_Matrix_dist(&Asave);
-	//	ScalePermstructFree(&ScalePermstruct);
+	//	dScalePermstructFree(&ScalePermstruct);
 	SUPERLU_FREE(b);
 	SUPERLU_FREE(bsave);
 	SUPERLU_FREE(xtrue);
